@@ -2,32 +2,56 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qplatformcamera_p.h"
+#include "private/qcameradevice_p.h"
 
 QT_BEGIN_NAMESPACE
 
-QPlatformCamera::QPlatformCamera(QCamera *parent)
-  : QObject(parent),
-    m_camera(parent)
+QPlatformCamera::QPlatformCamera(QCamera *parent) : QPlatformVideoSource(parent), m_camera(parent)
 {
+    qRegisterMetaType<QVideoFrame>();
 }
 
-QCameraFormat QPlatformCamera::findBestCameraFormat(const QCameraDevice &camera)
+QCameraFormat QPlatformCamera::findBestCameraFormat(const QCameraDevice &camera) const
 {
-    QCameraFormat f;
+    // check if fmt is better. We try to find the highest resolution that offers
+    // at least 30 FPS
+    // we use 29 FPS to compare against as some cameras report 29.97 FPS...
+
+    auto makeCriteria = [this](const QCameraFormat &fmt) {
+        constexpr float MinSufficientFrameRate = 29.f;
+
+        const auto isValid = fmt.pixelFormat() != QVideoFrameFormat::Format_Invalid;
+        const auto resolution = fmt.resolution();
+        const auto sufficientFrameRate = std::min(fmt.maxFrameRate(), MinSufficientFrameRate);
+        const auto pixelFormatScore =
+                cameraPixelFormatScore(fmt.pixelFormat(), QCameraFormatPrivate::getColorRange(fmt));
+
+        return std::make_tuple(
+                isValid, // 1st: ensure valid formats
+                sufficientFrameRate, // 2nd: ensure the highest frame rate in the range [0; 29]*/
+                resolution.width() * resolution.height(), // 3rd: ensure the highest resolution
+                pixelFormatScore, // 4th: eshure the best pixel format
+                fmt.maxFrameRate()); // 5th: ensure the highest framerate in the whole range
+    };
+
     const auto formats = camera.videoFormats();
-    for (const auto &fmt : formats) {
-        if (fmt.pixelFormat() == QVideoFrameFormat::Format_Invalid)
-            continue;
-        // check if fmt is better. We try to find the highest resolution that offers
-        // at least 30 FPS
-        // we use 29 FPS to compare against as some cameras report 29.97 FPS...
-        if (f.maxFrameRate() < 29 && fmt.maxFrameRate() > f.maxFrameRate())
-            f = fmt;
-        else if (f.maxFrameRate() == fmt.maxFrameRate() &&
-                 f.resolution().width()*f.resolution().height() < fmt.resolution().width()*fmt.resolution().height())
-            f = fmt;
-    }
-    return f;
+    const auto found =
+            std::max_element(formats.begin(), formats.end(),
+                             [makeCriteria](const QCameraFormat &fmtA, const QCameraFormat &fmtB) {
+                                 return makeCriteria(fmtA) < makeCriteria(fmtB);
+                             });
+
+    return found == formats.end() ? QCameraFormat{} : *found;
+}
+
+QVideoFrameFormat QPlatformCamera::frameFormat() const
+{
+    QVideoFrameFormat result(m_cameraFormat.resolution(),
+                             m_framePixelFormat == QVideoFrameFormat::Format_Invalid
+                                     ? m_cameraFormat.pixelFormat()
+                                     : m_framePixelFormat);
+    result.setStreamFrameRate(m_cameraFormat.maxFrameRate());
+    return result;
 }
 
 void QPlatformCamera::supportedFeaturesChanged(QCamera::Features f)
@@ -197,6 +221,13 @@ int QPlatformCamera::colorTemperatureForWhiteBalance(QCamera::WhiteBalanceMode m
     return 0;
 }
 
+void QPlatformCamera::updateError(QCamera::Error error, const QString &errorString)
+{
+    QMetaObject::invokeMethod(this, [this, error, errorString]() {
+        m_error.setAndNotify(error, errorString, *this);
+    });
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qplatformcamera_p.cpp"
