@@ -4,6 +4,8 @@
 #include "qvideoframeconversionhelper_p.h"
 #include "qrgb.h"
 
+#include <mutex>
+
 QT_BEGIN_NAMESPACE
 
 #define CLAMP(n) (n > 255 ? 255 : (n < 0 ? 0 : n))
@@ -32,31 +34,30 @@ static inline void planarYUV420_to_ARGB32(const uchar *y, int yStride,
                                           int width, int height)
 {
     height &= ~1;
-    quint32 *rgb0 = rgb;
-    quint32 *rgb1 = rgb + width;
 
-    for (int j = 0; j < height; j += 2) {
+    for (int j = 0; j + 1 < height; j += 2) {
         const uchar *lineY0 = y;
         const uchar *lineY1 = y + yStride;
         const uchar *lineU = u;
         const uchar *lineV = v;
 
-        for (int i = 0; i < width; i += 2) {
+        quint32 *rgb0 = rgb;
+        quint32 *rgb1 = rgb + width;
+        for (int i = 0; i + 1 < width; i += 2) {
             EXPAND_UV(*lineU, *lineV);
             lineU += uvPixelStride;
             lineV += uvPixelStride;
 
-            *rgb0++ = qYUVToARGB32(*lineY0++, rv, guv, bu);
-            *rgb0++ = qYUVToARGB32(*lineY0++, rv, guv, bu);
-            *rgb1++ = qYUVToARGB32(*lineY1++, rv, guv, bu);
-            *rgb1++ = qYUVToARGB32(*lineY1++, rv, guv, bu);
+            rgb0[i] = qYUVToARGB32(*lineY0++, rv, guv, bu);
+            rgb0[i + 1] = qYUVToARGB32(*lineY0++, rv, guv, bu);
+            rgb1[i] = qYUVToARGB32(*lineY1++, rv, guv, bu);
+            rgb1[i + 1] = qYUVToARGB32(*lineY1++, rv, guv, bu);
         }
 
         y += yStride << 1; // stride * 2
         u += uStride;
         v += vStride;
-        rgb0 += width;
-        rgb1 += width;
+        rgb += width << 1; // width * 2
     }
 }
 
@@ -67,31 +68,27 @@ static inline void planarYUV422_to_ARGB32(const uchar *y, int yStride,
                                           quint32 *rgb,
                                           int width, int height)
 {
-    quint32 *rgb0 = rgb;
-
     for (int j = 0; j < height; ++j) {
         const uchar *lineY0 = y;
         const uchar *lineU = u;
         const uchar *lineV = v;
 
-        for (int i = 0; i < width; i += 2) {
+        for (int i = 0; i + 1 < width; i += 2) {
             EXPAND_UV(*lineU, *lineV);
             lineU += uvPixelStride;
             lineV += uvPixelStride;
 
-            *rgb0++ = qYUVToARGB32(*lineY0++, rv, guv, bu);
-            *rgb0++ = qYUVToARGB32(*lineY0++, rv, guv, bu);
+            rgb[i]   = qYUVToARGB32(*lineY0++, rv, guv, bu);
+            rgb[i+1] = qYUVToARGB32(*lineY0++, rv, guv, bu);
         }
 
         y += yStride; // stride * 2
         u += uStride;
         v += vStride;
-        rgb0 += width;
+        rgb += width;
     }
 }
 
-
-
 static void QT_FASTCALL qt_convert_YUV420P_to_ARGB32(const QVideoFrame &frame, uchar *output)
 {
     FETCH_INFO_TRIPLANAR(frame)
@@ -185,8 +182,7 @@ static void QT_FASTCALL qt_convert_UYVY_to_ARGB32(const QVideoFrame &frame, ucha
 
     for (int i = 0; i < height; ++i) {
         const uchar *lineSrc = src;
-
-        for (int j = 0; j < width; j += 2) {
+        for (int j = 0; j + 1 < width; j += 2) {
             int u = *lineSrc++;
             int y0 = *lineSrc++;
             int v = *lineSrc++;
@@ -194,11 +190,12 @@ static void QT_FASTCALL qt_convert_UYVY_to_ARGB32(const QVideoFrame &frame, ucha
 
             EXPAND_UV(u, v);
 
-            *rgb++ = qYUVToARGB32(y0, rv, guv, bu);
-            *rgb++ = qYUVToARGB32(y1, rv, guv, bu);
+            rgb[j] = qYUVToARGB32(y0, rv, guv, bu);
+            rgb[j+1] = qYUVToARGB32(y1, rv, guv, bu);
         }
 
         src += stride;
+        rgb += width;
     }
 }
 
@@ -211,8 +208,7 @@ static void QT_FASTCALL qt_convert_YUYV_to_ARGB32(const QVideoFrame &frame, ucha
 
     for (int i = 0; i < height; ++i) {
         const uchar *lineSrc = src;
-
-        for (int j = 0; j < width; j += 2) {
+        for (int j = 0; j + 1 < width; j += 2) {
             int y0 = *lineSrc++;
             int u = *lineSrc++;
             int y1 = *lineSrc++;
@@ -220,11 +216,12 @@ static void QT_FASTCALL qt_convert_YUYV_to_ARGB32(const QVideoFrame &frame, ucha
 
             EXPAND_UV(u, v);
 
-            *rgb++ = qYUVToARGB32(y0, rv, guv, bu);
-            *rgb++ = qYUVToARGB32(y1, rv, guv, bu);
+            rgb[j] = qYUVToARGB32(y0, rv, guv, bu);
+            rgb[j+1] = qYUVToARGB32(y1, rv, guv, bu);
         }
 
         src += stride;
+        rgb += width;
     }
 }
 
@@ -374,23 +371,24 @@ static void QT_FASTCALL qt_convert_premultiplied_to_ARGB32(const QVideoFrame &fr
 }
 
 static inline void planarYUV420_16bit_to_ARGB32(const uchar *y, int yStride,
-                                                  const uchar *u, int uStride,
-                                                  const uchar *v, int vStride,
-                                                  int uvPixelStride,
-                                          quint32 *rgb,
-                                          int width, int height)
+                                                const uchar *u, int uStride,
+                                                const uchar *v, int vStride,
+                                                int uvPixelStride,
+                                                quint32 *rgb,
+                                                int width, int height)
 {
     height &= ~1;
-    quint32 *rgb0 = rgb;
-    quint32 *rgb1 = rgb + width;
 
-    for (int j = 0; j < height; j += 2) {
+    for (int j = 0; j + 1 < height; j += 2) {
         const uchar *lineY0 = y;
         const uchar *lineY1 = y + yStride;
         const uchar *lineU = u;
         const uchar *lineV = v;
 
-        for (int i = 0; i < width; i += 2) {
+        quint32 *rgb0 = rgb;
+        quint32 *rgb1 = rgb + width;
+
+        for (int i = 0; i + 1 < width; i += 2) {
             EXPAND_UV(*lineU, *lineV);
             lineU += uvPixelStride;
             lineV += uvPixelStride;
@@ -408,11 +406,11 @@ static inline void planarYUV420_16bit_to_ARGB32(const uchar *y, int yStride,
         y += yStride << 1; // stride * 2
         u += uStride;
         v += vStride;
-        rgb0 += width;
-        rgb1 += width;
+        rgb += width * 2;
     }
 }
 
+
 static void QT_FASTCALL qt_convert_P016_to_ARGB32(const QVideoFrame &frame, uchar *output)
 {
     FETCH_INFO_BIPLANAR(frame)
@@ -460,6 +458,14 @@ static void QT_FASTCALL qt_convert_Y_to_ARGB32(const QVideoFrame &frame, uchar *
     MERGE_LOOPS(width, height, stride, 1)
 }
 
+template<typename Pixel>
+static void QT_FASTCALL qt_copy_pixels_with_mask(Pixel *dst, const Pixel *src, size_t size,
+                                                 Pixel mask)
+{
+    for (size_t x = 0; x < size; ++x)
+        dst[x] = src[x] | mask;
+}
+
 static VideoFrameConvertFunc qConvertFuncs[QVideoFrameFormat::NPixelFormats] = {
     /* Format_Invalid */                nullptr, // Not needed
     /* Format_ARGB8888 */                 qt_convert_to_ARGB32<ARGB8888>,
@@ -492,24 +498,32 @@ static VideoFrameConvertFunc qConvertFuncs[QVideoFrameFormat::NPixelFormats] = {
     /* Format_Jpeg */                   nullptr, // Not needed
 };
 
-static void qInitConvertFuncsAsm()
+static PixelsCopyFunc qPixelsCopyFunc = qt_copy_pixels_with_mask<uint32_t>;
+
+static std::once_flag InitFuncsAsmFlag;
+
+static void qInitFuncsAsm()
 {
 #ifdef QT_COMPILER_SUPPORTS_SSE2
     extern void QT_FASTCALL  qt_convert_ARGB8888_to_ARGB32_sse2(const QVideoFrame &frame, uchar *output);
     extern void QT_FASTCALL  qt_convert_ABGR8888_to_ARGB32_sse2(const QVideoFrame &frame, uchar *output);
     extern void QT_FASTCALL  qt_convert_RGBA8888_to_ARGB32_sse2(const QVideoFrame &frame, uchar *output);
     extern void QT_FASTCALL  qt_convert_BGRA8888_to_ARGB32_sse2(const QVideoFrame &frame, uchar *output);
+    extern void QT_FASTCALL  qt_copy_pixels_with_mask_sse2(uint32_t * dst, const uint32_t *src, size_t size, uint32_t mask);
+
     if (qCpuHasFeature(SSE2)){
         qConvertFuncs[QVideoFrameFormat::Format_ARGB8888] = qt_convert_ARGB8888_to_ARGB32_sse2;
         qConvertFuncs[QVideoFrameFormat::Format_ARGB8888_Premultiplied] = qt_convert_ARGB8888_to_ARGB32_sse2;
         qConvertFuncs[QVideoFrameFormat::Format_XRGB8888] = qt_convert_ARGB8888_to_ARGB32_sse2;
         qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_sse2;
         qConvertFuncs[QVideoFrameFormat::Format_BGRA8888_Premultiplied] = qt_convert_BGRA8888_to_ARGB32_sse2;
-        qConvertFuncs[QVideoFrameFormat::Format_XBGR8888] = qt_convert_BGRA8888_to_ARGB32_sse2;
+        qConvertFuncs[QVideoFrameFormat::Format_BGRX8888] = qt_convert_BGRA8888_to_ARGB32_sse2;
         qConvertFuncs[QVideoFrameFormat::Format_ABGR8888] = qt_convert_ABGR8888_to_ARGB32_sse2;
         qConvertFuncs[QVideoFrameFormat::Format_XBGR8888] = qt_convert_ABGR8888_to_ARGB32_sse2;
-        qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_sse2;
-        qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_sse2;
+        qConvertFuncs[QVideoFrameFormat::Format_RGBA8888] = qt_convert_RGBA8888_to_ARGB32_sse2;
+        qConvertFuncs[QVideoFrameFormat::Format_RGBX8888] = qt_convert_RGBA8888_to_ARGB32_sse2;
+
+        qPixelsCopyFunc = qt_copy_pixels_with_mask_sse2;
     }
 #endif
 #ifdef QT_COMPILER_SUPPORTS_SSSE3
@@ -523,11 +537,11 @@ static void qInitConvertFuncsAsm()
         qConvertFuncs[QVideoFrameFormat::Format_XRGB8888] = qt_convert_ARGB8888_to_ARGB32_ssse3;
         qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_ssse3;
         qConvertFuncs[QVideoFrameFormat::Format_BGRA8888_Premultiplied] = qt_convert_BGRA8888_to_ARGB32_ssse3;
-        qConvertFuncs[QVideoFrameFormat::Format_XBGR8888] = qt_convert_BGRA8888_to_ARGB32_ssse3;
+        qConvertFuncs[QVideoFrameFormat::Format_BGRX8888] = qt_convert_BGRA8888_to_ARGB32_ssse3;
         qConvertFuncs[QVideoFrameFormat::Format_ABGR8888] = qt_convert_ABGR8888_to_ARGB32_ssse3;
         qConvertFuncs[QVideoFrameFormat::Format_XBGR8888] = qt_convert_ABGR8888_to_ARGB32_ssse3;
-        qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_ssse3;
-        qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_ssse3;
+        qConvertFuncs[QVideoFrameFormat::Format_RGBA8888] = qt_convert_RGBA8888_to_ARGB32_ssse3;
+        qConvertFuncs[QVideoFrameFormat::Format_RGBX8888] = qt_convert_RGBA8888_to_ARGB32_ssse3;
     }
 #endif
 #ifdef QT_COMPILER_SUPPORTS_AVX2
@@ -535,31 +549,82 @@ static void qInitConvertFuncsAsm()
     extern void QT_FASTCALL  qt_convert_ABGR8888_to_ARGB32_avx2(const QVideoFrame &frame, uchar *output);
     extern void QT_FASTCALL  qt_convert_RGBA8888_to_ARGB32_avx2(const QVideoFrame &frame, uchar *output);
     extern void QT_FASTCALL  qt_convert_BGRA8888_to_ARGB32_avx2(const QVideoFrame &frame, uchar *output);
+    extern void QT_FASTCALL  qt_copy_pixels_with_mask_avx2(uint32_t * dst, const uint32_t *src, size_t size, uint32_t mask);
     if (qCpuHasFeature(AVX2)){
         qConvertFuncs[QVideoFrameFormat::Format_ARGB8888] = qt_convert_ARGB8888_to_ARGB32_avx2;
         qConvertFuncs[QVideoFrameFormat::Format_ARGB8888_Premultiplied] = qt_convert_ARGB8888_to_ARGB32_avx2;
         qConvertFuncs[QVideoFrameFormat::Format_XRGB8888] = qt_convert_ARGB8888_to_ARGB32_avx2;
         qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_avx2;
         qConvertFuncs[QVideoFrameFormat::Format_BGRA8888_Premultiplied] = qt_convert_BGRA8888_to_ARGB32_avx2;
-        qConvertFuncs[QVideoFrameFormat::Format_XBGR8888] = qt_convert_BGRA8888_to_ARGB32_avx2;
+        qConvertFuncs[QVideoFrameFormat::Format_BGRX8888] = qt_convert_BGRA8888_to_ARGB32_avx2;
         qConvertFuncs[QVideoFrameFormat::Format_ABGR8888] = qt_convert_ABGR8888_to_ARGB32_avx2;
         qConvertFuncs[QVideoFrameFormat::Format_XBGR8888] = qt_convert_ABGR8888_to_ARGB32_avx2;
-        qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_avx2;
-        qConvertFuncs[QVideoFrameFormat::Format_BGRA8888] = qt_convert_BGRA8888_to_ARGB32_avx2;
+        qConvertFuncs[QVideoFrameFormat::Format_RGBA8888] = qt_convert_RGBA8888_to_ARGB32_avx2;
+        qConvertFuncs[QVideoFrameFormat::Format_RGBX8888] = qt_convert_RGBA8888_to_ARGB32_avx2;
+
+        qPixelsCopyFunc = qt_copy_pixels_with_mask_avx2;
     }
 #endif
 }
 
 VideoFrameConvertFunc qConverterForFormat(QVideoFrameFormat::PixelFormat format)
 {
-    static bool initAsmFuncsDone = false;
-    if (!initAsmFuncsDone) {
-        qInitConvertFuncsAsm();
-        initAsmFuncsDone = true;
-    }
+    std::call_once(InitFuncsAsmFlag, &qInitFuncsAsm);
+
     VideoFrameConvertFunc convert = qConvertFuncs[format];
     return convert;
 }
 
+void Q_MULTIMEDIA_EXPORT qCopyPixelsWithAlphaMask(uint32_t *dst,
+                                                  const uint32_t *src,
+                                                  size_t pixCount,
+                                                  QVideoFrameFormat::PixelFormat format,
+                                                  bool srcAlphaVaries)
+{
+    if (pixCount == 0)
+        return;
+
+    const auto mask = qAlphaMask(format);
+
+    if (srcAlphaVaries || (src[0] & mask) != mask)
+        qCopyPixelsWithMask(dst, src, pixCount, mask);
+    else
+        memcpy(dst, src, pixCount * 4);
+}
+
+void qCopyPixelsWithMask(uint32_t *dst, const uint32_t *src, size_t size, uint32_t mask)
+{
+    std::call_once(InitFuncsAsmFlag, &qInitFuncsAsm);
+
+    qPixelsCopyFunc(dst, src, size, mask);
+}
+
+uint32_t qAlphaMask(QVideoFrameFormat::PixelFormat format)
+{
+    switch (format) {
+    case QVideoFrameFormat::Format_ARGB8888:
+    case QVideoFrameFormat::Format_ARGB8888_Premultiplied:
+    case QVideoFrameFormat::Format_XRGB8888:
+    case QVideoFrameFormat::Format_ABGR8888:
+    case QVideoFrameFormat::Format_XBGR8888:
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        return 0xff;
+#else
+        return 0xff000000;
+#endif
+    case QVideoFrameFormat::Format_BGRA8888:
+    case QVideoFrameFormat::Format_BGRA8888_Premultiplied:
+    case QVideoFrameFormat::Format_BGRX8888:
+    case QVideoFrameFormat::Format_RGBA8888:
+    case QVideoFrameFormat::Format_RGBX8888:
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        return 0xff000000;
+#else
+        return 0xff;
+#endif
+    default:
+        return 0;
+    }
+}
 
 QT_END_NAMESPACE
