@@ -23,7 +23,7 @@ QWasmCameraDevices::QWasmCameraDevices(QPlatformMediaIntegration *integration)
 {
 }
 
-QList<QCameraDevice> QWasmCameraDevices::videoInputs() const
+QList<QCameraDevice> QWasmCameraDevices::findVideoInputs() const
 {
     return m_mediaDevices ? m_mediaDevices->videoInputs() : QList<QCameraDevice>();
 }
@@ -59,15 +59,17 @@ QList<QCameraDevice> QWasmMediaDevices::videoInputs() const
 }
 
 QPlatformAudioSource *QWasmMediaDevices::createAudioSource(const QAudioDevice &deviceInfo,
+                                                           const QAudioFormat &fmt,
                                                            QObject *parent)
 {
-    return new QWasmAudioSource(deviceInfo.id(), parent);
+    return new QWasmAudioSource(deviceInfo, fmt, parent);
 }
 
 QPlatformAudioSink *QWasmMediaDevices::createAudioSink(const QAudioDevice &deviceInfo,
+                                                       const QAudioFormat &fmt,
                                                        QObject *parent)
 {
-    return new QWasmAudioSink(deviceInfo.id(), parent);
+    return new QWasmAudioSink(deviceInfo, fmt, parent);
 }
 
 void QWasmMediaDevices::parseDevices(emscripten::val devices)
@@ -127,10 +129,10 @@ void QWasmMediaDevices::parseDevices(emscripten::val devices)
         } else if (deviceKind == std::string("audioinput")) {
             if (!m_audioInputs.contains(deviceId)) {
                 isDefault = !m_audioInputsAdded;
-                m_audioInputs.insert(deviceId,
-                                     (new QWasmAudioDevice(deviceId.c_str(), label.c_str(),
-                                                           isDefault, QAudioDevice::Input))
-                                             ->create());
+                m_audioInputs.insert(
+                        deviceId,
+                        QAudioDevicePrivate::createQAudioDevice(std::make_unique<QWasmAudioDevice>(
+                                deviceId.c_str(), label.c_str(), isDefault, QAudioDevice::Input)));
 
                 m_audioInputsAdded = true;
             }
@@ -138,10 +140,11 @@ void QWasmMediaDevices::parseDevices(emscripten::val devices)
         } else if (deviceKind == std::string("audiooutput")) {
             if (!m_audioOutputs.contains(deviceId)) {
                 isDefault = !m_audioOutputsAdded;
-                m_audioOutputs.insert(deviceId,
-                                      (new QWasmAudioDevice(deviceId.c_str(), label.c_str(),
-                                                            isDefault, QAudioDevice::Input))
-                                              ->create());
+                m_audioOutputs.insert(
+                        deviceId,
+                        QAudioDevicePrivate::createQAudioDevice(std::make_unique<QWasmAudioDevice>(
+                                deviceId.c_str(), label.c_str(), isDefault, QAudioDevice::Output)));
+                ;
 
                 m_audioOutputsAdded = true;
             }
@@ -170,8 +173,10 @@ void QWasmMediaDevices::parseDevices(emscripten::val devices)
     }
     m_audioOutputsRemoved = !audioOutputsToRemove.isEmpty();
 
-    if (m_videoInputsAdded || m_videoInputsRemoved)
-        emit QPlatformMediaIntegration::instance()->videoDevices()->videoInputsChanged();
+    if (m_videoInputsAdded || m_videoInputsRemoved) {
+        auto videoDevices = static_cast<QWasmCameraDevices*>(QPlatformMediaIntegration::instance()->videoDevices());
+        videoDevices->onVideoInputsChanged();
+    }
     if (m_audioInputsAdded || m_audioInputsRemoved)
         onAudioInputsChanged();
     if (m_audioOutputsAdded || m_audioOutputsRemoved)
@@ -239,10 +244,10 @@ void QWasmMediaDevices::getOpenALAudioDevices()
     auto capture = alcGetString(nullptr, ALC_CAPTURE_DEFAULT_DEVICE_SPECIFIER);
     // present even if there is no capture device
     if (capture && !m_audioOutputs.contains(capture)) {
-        m_audioInputs.insert(capture,
-                             (new QWasmAudioDevice(capture, "WebAssembly audio capture device",
-                                                   true, QAudioDevice::Input))
-                                     ->create());
+        m_audioInputs.insert(
+                capture,
+                QAudioDevicePrivate::createQAudioDevice(std::make_unique<QWasmAudioDevice>(
+                        capture, "WebAssembly audio capture device", true, QAudioDevice::Input)));
         m_audioInputsAdded = true;
         onAudioInputsChanged();
     }
@@ -250,10 +255,11 @@ void QWasmMediaDevices::getOpenALAudioDevices()
     auto playback = alcGetString(nullptr, ALC_DEFAULT_DEVICE_SPECIFIER);
     // present even if there is no playback device
     if (playback && !m_audioOutputs.contains(capture)) {
-        m_audioOutputs.insert(playback,
-                              (new QWasmAudioDevice(playback, "WebAssembly audio playback device",
-                                                    true, QAudioDevice::Output))
-                                      ->create());
+        m_audioOutputs.insert(
+                playback,
+                QAudioDevicePrivate::createQAudioDevice(std::make_unique<QWasmAudioDevice>(
+                        playback, "WebAssembly audio playback device", true,
+                        QAudioDevice::Output)));
         onAudioOutputsChanged();
     }
     m_firstInit = true;
