@@ -5,7 +5,9 @@
 
 #include <QtCore/qthread.h>
 #include <QtCore/qloggingcategory.h>
+#include <QtCore/private/qfunctions_win_p.h>
 #include <QtCore/private/quniquehandle_types_p.h>
+#include <QtMultimedia/private/qaudioformat_p.h>
 #include <QtMultimedia/private/qaudiosystem_platform_stream_support_p.h>
 #include <QtMultimedia/private/qwindowsaudiodevice_p.h>
 #include <QtMultimedia/private/qwindowsaudioutils_p.h>
@@ -15,338 +17,47 @@
 
 QT_BEGIN_NAMESPACE
 
-using QtMultimediaPrivate::QPlatformAudioIOStream;
-using QtMultimediaPrivate::QPlatformAudioSourceStream;
+namespace QtWASAPI {
+
 using QWindowsAudioUtils::audioClientErrorString;
 using namespace std::chrono_literals;
 
-struct QWASAPIAudioSourceStream final : std::enable_shared_from_this<QWASAPIAudioSourceStream>,
-                                        QPlatformAudioSourceStream
+namespace {
+QAudioFormat makeHostFormatForSource(const QAudioDevice &device, const QAudioFormat &format)
 {
-    using SampleFormat = QAudioFormat::SampleFormat;
+    const QWindowsAudioDevice *winDevice = QAudioDevicePrivate::handle<QWindowsAudioDevice>(device);
 
-    QWASAPIAudioSourceStream(QAudioDevice, QWindowsAudioSource *parent, const QAudioFormat &,
-                             std::optional<qsizetype> ringbufferSize,
-                             std::optional<int32_t> hardwareBufferFrames, float volume);
+    QAudioFormat hostFormat = format;
+    const int requestedChannelCount = format.channelCount();
+    auto [minProbedChannels, maxProbedChannels] = winDevice->m_probedChannelCountRange;
 
-    ~QWASAPIAudioSourceStream();
-
-    using QPlatformAudioSourceStream::bytesReady;
-    using QPlatformAudioSourceStream::deviceIsRingbufferReader;
-    using QPlatformAudioSourceStream::processedDuration;
-    using QPlatformAudioSourceStream::ringbufferSizeInBytes;
-    using QPlatformAudioSourceStream::setVolume;
-
-    bool start(QIODevice *, ComPtr<IMMDevice>);
-    QIODevice *start(ComPtr<IMMDevice>);
-
-    void suspend();
-    void resume();
-    void stop(ShutdownPolicy);
-
-    void updateStreamIdle(bool) override;
-
-private:
-    bool openAudioClient(ComPtr<IMMDevice> device);
-    bool startAudioClient();
-
-    void runProcessLoop();
-    bool process() noexcept QT_MM_NONBLOCKING;
-    void handleAudioClientError();
-
-    ComPtr<IAudioClient3> m_audioClient;
-    ComPtr<IAudioCaptureClient> m_captureClient;
-
-    QWindowsAudioUtils::reference_time m_periodSize;
-    qsizetype m_audioClientFrames;
-
-    std::atomic_bool m_suspended{};
-    std::atomic<ShutdownPolicy> m_shutdownPolicy{ ShutdownPolicy::DiscardRingbuffer };
-    QAutoResetEvent m_ringbufferDrained;
-
-    const QUniqueWin32NullHandle m_wasapiHandle;
-    std::unique_ptr<QThread> m_workerThread;
-
-    QWindowsAudioSource *m_parent;
-};
-
-QWASAPIAudioSourceStream::~QWASAPIAudioSourceStream() = default;
-
-void QWASAPIAudioSourceStream::updateStreamIdle(bool streamIsIdle)
-{
-    if (m_parent)
-        m_parent->updateStreamIdle(streamIsIdle);
-}
-
-bool QWASAPIAudioSourceStream::openAudioClient(ComPtr<IMMDevice> device)
-{
-    using namespace QWindowsAudioUtils;
-
-    std::optional<AudioClientCreationResult> clientData =
-            createAudioClient(device, m_format, m_hardwareBufferFrames, m_wasapiHandle);
-
-    if (!clientData)
-        return false;
-
-    m_audioClient = std::move(clientData->client);
-    m_periodSize = clientData->periodSize;
-    m_audioClientFrames = clientData->audioClientFrames;
-
-    HRESULT hr = m_audioClient->GetService(IID_PPV_ARGS(m_captureClient.GetAddressOf()));
-    if (FAILED(hr)) {
-        qWarning() << "IAudioClient3::GetService failed to obtain IAudioCaptureClient"
-                   << audioClientErrorString(hr);
-        return false;
+    if (requestedChannelCount < device.minimumChannelCount()) {
+        hostFormat.setChannelCount(minProbedChannels);
+        hostFormat.setChannelConfig(
+                QAudioFormat::defaultChannelConfigForChannelCount(minProbedChannels));
+    } else if (requestedChannelCount > device.maximumChannelCount()) {
+        hostFormat.setChannelCount(maxProbedChannels);
+        hostFormat.setChannelConfig(
+                QAudioFormat::defaultChannelConfigForChannelCount(maxProbedChannels));
     }
 
-    if (m_audioDevice.preferredFormat().sampleRate() != m_format.sampleRate())
-        audioClientSetRate(m_audioClient, m_format.sampleRate());
+    const int requestedSampleRate = format.sampleRate();
+    auto [minProbedSampleRate, maxProbedSampleRate] = winDevice->m_probedSampleRateRange;
 
-    return true;
+    if (requestedSampleRate < device.minimumSampleRate())
+        hostFormat.setSampleRate(minProbedSampleRate);
+    else if (requestedSampleRate > device.maximumSampleRate())
+        hostFormat.setSampleRate(maxProbedSampleRate);
+
+    return hostFormat;
 }
+} // namespace
 
-bool QWASAPIAudioSourceStream::startAudioClient()
-{
-    using namespace QWindowsAudioUtils;
-    m_workerThread.reset(QThread::create([this] {
-        setMCSSForPeriodSize(m_periodSize);
-        runProcessLoop();
-    }));
-
-    m_workerThread->setObjectName(u"QWASAPIAudioSourceStream");
-    m_workerThread->start();
-
-    return audioClientStart(m_audioClient);
-}
-
-void QWASAPIAudioSourceStream::runProcessLoop()
-{
-    using namespace QWindowsAudioUtils;
-    for (;;) {
-        constexpr std::chrono::milliseconds timeout = 2s;
-        DWORD retval = WaitForSingleObject(m_wasapiHandle.get(), timeout.count());
-        if (retval != WAIT_OBJECT_0) {
-            if (m_suspended)
-                continue;
-
-            handleAudioClientError();
-            return;
-        }
-
-        if (isStopRequested())
-            return; // TODO: distinguish between stop/reset?
-
-        bool success = process();
-        if (!success) {
-            handleAudioClientError();
-            return;
-        }
-    }
-}
-
-bool QWASAPIAudioSourceStream::process() noexcept QT_MM_NONBLOCKING
-{
-    for (;;) {
-        unsigned char *hostBuffer;
-        uint32_t hostBufferFrames;
-        DWORD flags;
-        uint64_t devicePosition;
-        uint64_t QPCPosition;
-        HRESULT hr = m_captureClient->GetBuffer(&hostBuffer, &hostBufferFrames, &flags,
-                                                &devicePosition, &QPCPosition);
-        if (FAILED(hr)) {
-            qWarning() << "IAudioCaptureClient::GetBuffer failed" << audioClientErrorString(hr);
-            return false;
-        }
-
-        QSpan hostBufferSpan{
-            hostBuffer,
-            m_format.bytesForFrames(hostBufferFrames),
-        };
-
-        uint64_t framesWritten =
-                QPlatformAudioSourceStream::process(as_bytes(hostBufferSpan), hostBufferFrames);
-        if (framesWritten != hostBufferFrames)
-            updateStreamIdle(true);
-
-        hr = m_captureClient->ReleaseBuffer(hostBufferFrames);
-        if (FAILED(hr)) {
-            qWarning() << "IAudioCaptureClient::ReleaseBuffer failed" << audioClientErrorString(hr);
-            return false;
-        }
-
-        uint32_t framesInNextPacket;
-        hr = m_captureClient->GetNextPacketSize(&framesInNextPacket);
-
-        if (FAILED(hr)) {
-            qWarning() << "IAudioCaptureClient::GetNextPacketSize failed"
-                       << audioClientErrorString(hr);
-            return false;
-        }
-
-        if (framesInNextPacket == 0)
-            return true;
-    }
-}
-
-void QWASAPIAudioSourceStream::handleAudioClientError()
-{
-    using namespace QWindowsAudioUtils;
-    audioClientStop(m_audioClient);
-    audioClientReset(m_audioClient);
-
-    QMetaObject::invokeMethod(&m_ringbufferDrained, [this] {
-        handleIOError(m_parent);
-    });
-}
-
-///////////////////////////////////////////////////////////////////////////////////////////////////
-
-QWindowsAudioSource::QWindowsAudioSource(QAudioDevice audioDevice, const QAudioFormat &fmt,
-                                         QObject *parent)
-    : QPlatformAudioSource(std::move(audioDevice), fmt, parent)
-{
-}
-
-QWindowsAudioSource::~QWindowsAudioSource()
-{
-    stop();
-}
-
-QIODevice *QWindowsAudioSource::start()
-{
-    auto immDevice = QAudioDevicePrivate::handle<QWindowsAudioDevice>(m_audioDevice)->open();
-    if (!immDevice) {
-        setError(QtAudio::OpenError);
-        m_stream = {};
-        return nullptr;
-    }
-
-    m_stream = std::make_unique<QWASAPIAudioSourceStream>(
-            m_audioDevice, this, m_format, m_bufferSize, m_hardwareBufferFrames, volume());
-
-    QIODevice *ioDevice = m_stream->start(std::move(immDevice));
-    if (!ioDevice) {
-        setError(QtAudio::OpenError);
-        return nullptr;
-    }
-
-    updateStreamState(QtAudio::State::ActiveState);
-
-    return ioDevice;
-}
-
-void QWindowsAudioSource::start(QIODevice *iodevice)
-{
-    auto immDevice = QAudioDevicePrivate::handle<QWindowsAudioDevice>(m_audioDevice)->open();
-    if (!immDevice) {
-        setError(QtAudio::OpenError);
-        return;
-    }
-
-    m_stream = std::make_unique<QWASAPIAudioSourceStream>(
-            m_audioDevice, this, m_format, m_bufferSize, m_hardwareBufferFrames, volume());
-
-    bool started = m_stream->start(iodevice, std::move(immDevice));
-    if (!started) {
-        setError(QtAudio::OpenError);
-        m_stream = {};
-        return;
-    }
-
-    updateStreamState(QtAudio::State::ActiveState);
-}
-
-void QWindowsAudioSource::stop()
-{
-    if (!m_stream)
-        return;
-
-    if (m_stream->deviceIsRingbufferReader())
-        // we own the qiodevice, so let's keep it alive to allow users to drain the ringbuffer
-        m_retiredStream = m_stream;
-
-    m_stream->stop(QPlatformAudioIOStream::ShutdownPolicy::DrainRingbuffer);
-    m_stream = {};
-    updateStreamState(QtAudio::State::StoppedState);
-}
-
-void QWindowsAudioSource::reset()
-{
-    m_retiredStream = {};
-
-    if (!m_stream)
-        return;
-
-    m_stream->stop(QPlatformAudioIOStream::ShutdownPolicy::DiscardRingbuffer);
-    m_stream = {};
-    updateStreamState(QtAudio::State::StoppedState);
-}
-
-void QWindowsAudioSource::suspend()
-{
-    if (!m_stream)
-        return;
-
-    m_stream->suspend();
-
-    updateStreamState(QtAudio::State::SuspendedState);
-}
-
-void QWindowsAudioSource::resume()
-{
-    if (!m_stream)
-        return;
-
-    if (state() == QtAudio::State::ActiveState)
-        return;
-
-    m_stream->resume();
-
-    updateStreamState(QtAudio::State::ActiveState);
-}
-
-qsizetype QWindowsAudioSource::bytesReady() const
-{
-    return m_stream ? m_stream->bytesReady() : 0;
-}
-
-void QWindowsAudioSource::setBufferSize(qsizetype value)
-{
-    m_bufferSize = value;
-}
-
-qsizetype QWindowsAudioSource::bufferSize() const
-{
-    if (m_stream)
-        return m_stream->ringbufferSizeInBytes();
-
-    return QPlatformAudioIOStream::inferRingbufferBytes(m_bufferSize, m_hardwareBufferFrames,
-                                                        m_format);
-}
-
-void QWindowsAudioSource::setHardwareBufferFrames(int32_t arg)
-{
-    if (arg > 0)
-        m_hardwareBufferFrames = arg;
-    else
-        m_hardwareBufferFrames = std::nullopt;
-}
-
-int32_t QWindowsAudioSource::hardwareBufferFrames()
-{
-    return m_hardwareBufferFrames.value_or(-1);
-}
-
-qint64 QWindowsAudioSource::processedUSecs() const
-{
-    return m_stream ? m_stream->processedDuration().count() : 0;
-}
-
-QWASAPIAudioSourceStream::QWASAPIAudioSourceStream(QAudioDevice device, QWindowsAudioSource *parent,
-                                                   const QAudioFormat &format,
+QWASAPIAudioSourceStream::QWASAPIAudioSourceStream(QAudioDevice device, const QAudioFormat &format,
                                                    std::optional<qsizetype> ringbufferSize,
-                                                   std::optional<int32_t> hardwareBufferFrames, float volume):
+                                                   QWindowsAudioSource *parent,
+                                                   float volume,
+                                                   std::optional<int32_t> hardwareBufferFrames):
     QPlatformAudioSourceStream{
         std::move(device),
         format,
@@ -359,12 +70,19 @@ QWASAPIAudioSourceStream::QWASAPIAudioSourceStream(QAudioDevice device, QWindows
     },
     m_parent{
         parent
+    },
+    m_hostFormat {
+        makeHostFormatForSource(m_audioDevice, format),
     }
 {
 }
 
-bool QWASAPIAudioSourceStream::start(QIODevice *ioDevice, ComPtr<IMMDevice> immDevice)
+QWASAPIAudioSourceStream::~QWASAPIAudioSourceStream() = default;
+
+bool QWASAPIAudioSourceStream::start(QIODevice *ioDevice)
 {
+    auto immDevice = QAudioDevicePrivate::handle<QWindowsAudioDevice>(m_audioDevice)->open();
+
     bool clientOpen = openAudioClient(std::move(immDevice));
     if (!clientOpen)
         return false;
@@ -379,8 +97,10 @@ bool QWASAPIAudioSourceStream::start(QIODevice *ioDevice, ComPtr<IMMDevice> immD
     return true;
 }
 
-QIODevice *QWASAPIAudioSourceStream::start(ComPtr<IMMDevice> immDevice)
+QIODevice *QWASAPIAudioSourceStream::start()
 {
+    auto immDevice = QAudioDevicePrivate::handle<QWindowsAudioDevice>(m_audioDevice)->open();
+
     bool clientOpen = openAudioClient(std::move(immDevice));
     if (!clientOpen)
         return nullptr;
@@ -399,6 +119,19 @@ QIODevice *QWASAPIAudioSourceStream::start(ComPtr<IMMDevice> immDevice)
     return ioDevice;
 }
 
+bool QWASAPIAudioSourceStream::start(AudioCallback &&cb)
+{
+    auto immDevice = QAudioDevicePrivate::handle<QWindowsAudioDevice>(m_audioDevice)->open();
+
+    bool clientOpen = openAudioClient(std::move(immDevice));
+    if (!clientOpen)
+        return false;
+
+    m_audioCallback = std::move(cb);
+
+    return startAudioClient();
+}
+
 void QWASAPIAudioSourceStream::suspend()
 {
     m_suspended = true;
@@ -428,11 +161,178 @@ void QWASAPIAudioSourceStream::stop(ShutdownPolicy shutdownPolicy)
         emptyRingbuffer();
 }
 
-void QWindowsAudioSource::setVolume(float v)
+void QWASAPIAudioSourceStream::updateStreamIdle(bool streamIsIdle)
 {
-    QPlatformAudioEndpointBase::setVolume(v);
-    if (m_stream)
-        m_stream->setVolume(v);
+    if (m_parent)
+        m_parent->updateStreamIdle(streamIsIdle);
 }
 
+bool QWASAPIAudioSourceStream::openAudioClient(ComPtr<IMMDevice> device)
+{
+    using namespace QWindowsAudioUtils;
+
+    std::optional<AudioClientCreationResult> clientData =
+            createAudioClient(device, m_hostFormat, m_hardwareBufferFrames, m_wasapiHandle);
+
+    if (!clientData)
+        return false;
+
+    m_audioClient = std::move(clientData->client);
+    m_periodSize = clientData->periodSize;
+    m_audioClientFrames = clientData->audioClientFrames;
+
+    HRESULT hr = m_audioClient->GetService(IID_PPV_ARGS(m_captureClient.GetAddressOf()));
+    if (FAILED(hr)) {
+        qWarning() << "IAudioClient3::GetService failed to obtain IAudioCaptureClient"
+                   << audioClientErrorString(hr);
+        return false;
+    }
+
+    return true;
+}
+
+bool QWASAPIAudioSourceStream::startAudioClient()
+{
+    using namespace QWindowsAudioUtils;
+    m_workerThread.reset(QThread::create([this] {
+        setMCSSForPeriodSize(m_periodSize);
+
+        std::optional<QComHelper> m_comHelper;
+
+        if (m_hostFormat != m_format) {
+            m_comHelper.emplace();
+            m_resampler = std::make_unique<QWindowsResampler>();
+            m_resampler->setup(m_hostFormat, m_format);
+
+            m_preallocatedBuffer = std::make_unique<char[]>(512 * 1024); // 512 KiB
+
+            m_pmrBufferResource = std::make_unique<std::pmr::monotonic_buffer_resource>(
+                    m_preallocatedBuffer.get(), 512 * 1024, std::pmr::get_default_resource());
+
+            std::pmr::pool_options poolOptions{
+                /*.largest_required_pool_block =*/256 * 1024,
+                /*.min_blocks_per_chunk        =*/2,
+            };
+            m_pmrPoolResource = std::make_unique<std::pmr::unsynchronized_pool_resource>(
+                    poolOptions, m_pmrBufferResource.get());
+        }
+
+        runProcessLoop();
+    }));
+
+    m_workerThread->setObjectName(u"QWASAPIAudioSourceStream");
+    m_workerThread->start();
+
+    return audioClientStart(m_audioClient);
+}
+
+void QWASAPIAudioSourceStream::runProcessLoop()
+{
+    using namespace QWindowsAudioUtils;
+    for (;;) {
+        constexpr std::chrono::milliseconds timeout = 2s;
+        DWORD retval = WaitForSingleObject(m_wasapiHandle.get(), timeout.count());
+        if (retval != WAIT_OBJECT_0) {
+            if (m_suspended)
+                continue;
+
+            handleAudioClientError();
+            return;
+        }
+
+        if (isStopRequested())
+            return; // TODO: distinguish between stop/reset?
+
+        bool success = m_audioCallback ? processCallback() : processRingbuffer();
+        if (!success) {
+            handleAudioClientError();
+            return;
+        }
+    }
+}
+
+template <typename Functor>
+bool QWASAPIAudioSourceStream::visitAudioClientBuffer(Functor &&f)
+{
+    for (;;) {
+        unsigned char *hostBuffer;
+        uint32_t hostBufferFrames;
+        DWORD flags;
+        uint64_t devicePosition;
+        uint64_t QPCPosition;
+        HRESULT hr = m_captureClient->GetBuffer(&hostBuffer, &hostBufferFrames, &flags,
+                                                &devicePosition, &QPCPosition);
+        if (FAILED(hr)) {
+            qWarning() << "IAudioCaptureClient::GetBuffer failed" << audioClientErrorString(hr);
+            return false;
+        }
+
+        // Note: when starting the capture client, we an initial wakeup, despite no buffer being
+        // available
+        if (hostBufferFrames == 0)
+            return true;
+
+        QSpan hostBufferSpan{
+            hostBuffer,
+            m_hostFormat.bytesForFrames(hostBufferFrames),
+        };
+
+        if (m_resampler) {
+            Q_UNLIKELY_BRANCH;
+            auto resampledBuffer =
+                    m_resampler->resample(as_bytes(hostBufferSpan), m_pmrPoolResource.get());
+            QPlatformAudioSourceStream::process(resampledBuffer,
+                                                m_format.framesForBytes(resampledBuffer.size()));
+        } else {
+            f(as_bytes(hostBufferSpan), hostBufferFrames);
+        }
+
+        hr = m_captureClient->ReleaseBuffer(hostBufferFrames);
+
+        if (FAILED(hr)) {
+            qWarning() << "IAudioCaptureClient::ReleaseBuffer failed" << audioClientErrorString(hr);
+            return false;
+        }
+    }
+}
+
+bool QWASAPIAudioSourceStream::processRingbuffer() noexcept QT_MM_NONBLOCKING
+{
+    return visitAudioClientBuffer(
+            [&](QSpan<const std::byte> hostBuffer, uint32_t hostBufferFrames) {
+        uint64_t framesWritten = QPlatformAudioSourceStream::process(hostBuffer, hostBufferFrames);
+        if (framesWritten != hostBufferFrames)
+            updateStreamIdle(true);
+    });
+}
+
+bool QWASAPIAudioSourceStream::processCallback() noexcept QT_MM_NONBLOCKING
+{
+    return visitAudioClientBuffer([&](QSpan<const std::byte> hostBuffer, uint32_t) {
+        runAudioCallback(*m_audioCallback, as_bytes(hostBuffer), m_format);
+    });
+}
+
+void QWASAPIAudioSourceStream::handleAudioClientError()
+{
+    using namespace QWindowsAudioUtils;
+    audioClientStop(m_audioClient);
+    audioClientReset(m_audioClient);
+    requestStop();
+
+    QMetaObject::invokeMethod(&m_ringbufferDrained, [this] {
+        handleIOError(m_parent);
+    });
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+
+QWindowsAudioSource::QWindowsAudioSource(QAudioDevice audioDevice, const QAudioFormat &fmt,
+                                         QObject *parent)
+    : BaseClass(std::move(audioDevice), fmt, parent)
+{
+}
+
+} // namespace QtWASAPI
+
 QT_END_NAMESPACE
