@@ -42,7 +42,7 @@ class AVFMediaPlayer : public QObject, public QPlatformMediaPlayer
     Q_OBJECT
 public:
     AVFMediaPlayer(QMediaPlayer *parent);
-    virtual ~AVFMediaPlayer();
+    ~AVFMediaPlayer() override;
 
     void setVideoSink(QVideoSink *sink) override;
     void setVideoOutput(AVFVideoRendererControl *output);
@@ -74,7 +74,7 @@ public:
     QMediaMetaData metaData() const override;
 
     static void videoOrientationForAssetTrack(AVAssetTrack *track,
-                                       QVideo::RotationAngle &angle,
+                                       QtVideo::Rotation &angle,
                                        bool &mirrored);
 
 public Q_SLOTS:
@@ -89,7 +89,7 @@ public Q_SLOTS:
 
     void setVolume(float volume);
     void setMuted(bool muted);
-    void audioOutputChanged();
+    void updateAudioOutputDevice();
 
     void processEOS();
     void processLoadStateChange(QMediaPlayer::PlaybackState newState);
@@ -121,7 +121,7 @@ private:
     void setSeekable(bool seekable);
     void resetStream(QIODevice *stream = nullptr);
 
-    void orientationChanged(QVideo::RotationAngle rotation, bool mirrored);
+    void orientationChanged(QtVideo::Rotation rotation, bool mirrored);
 
     AVFVideoRendererControl *m_videoOutput = nullptr;
     AVFVideoSink *m_videoSink = nullptr;
