@@ -1,8 +1,8 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#ifndef QFANDROIDVIDEODEVICES_H
-#define QFANDROIDVIDEODEVICES_H
+#ifndef QANDROIDVIDEODEVICES_H
+#define QANDROIDVIDEODEVICES_H
 
 //
 //  W A R N I N G
@@ -16,20 +16,30 @@
 //
 
 #include <QObject>
-#include <private/qplatformvideodevices_p.h>
+#include <QJniObject>
+
+#include <QtMultimedia/private/qplatformvideodevices_p.h>
+
+QT_BEGIN_NAMESPACE
 
 class QAndroidVideoDevices : public QPlatformVideoDevices
 {
     Q_OBJECT
 public:
-    QAndroidVideoDevices(QPlatformMediaIntegration *integration)
-        : QPlatformVideoDevices(integration), m_videoDevices(findVideoDevices()){};
+    QAndroidVideoDevices(QPlatformMediaIntegration *integration);
+    ~QAndroidVideoDevices() override;
 
-    QList<QCameraDevice> videoInputs() const override { return m_videoDevices; }
+    using QPlatformVideoDevices::onVideoInputsChanged;
+
+protected:
+    QList<QCameraDevice> findVideoInputs() const override;
 
 private:
-    QList<QCameraDevice> findVideoDevices();
-    QList<QCameraDevice> m_videoDevices;
+    QJniObject m_javaCameraAvailabilityListener;
+
+    static void registerNativeMethods();
 };
 
-#endif // QFANDROIDVIDEODEVICES_H
+QT_END_NAMESPACE
+
+#endif // QANDROIDVIDEODEVICES_H
