@@ -33,7 +33,7 @@ Q_DECLARE_JNI_CLASS(QtVideoDeviceManager,
                     "org/qtproject/qt/android/multimedia/QtVideoDeviceManager")
 #endif // Q_OS_ANDROID
 
-static Q_LOGGING_CATEGORY(qLcCodecStorage, "qt.multimedia.ffmpeg.codecstorage");
+Q_STATIC_LOGGING_CATEGORY(qLcCodecStorage, "qt.multimedia.ffmpeg.codecstorage");
 
 namespace QFFmpeg {
 
@@ -99,7 +99,9 @@ void dumpCodecInfo(const Codec &codec)
         { AV_CODEC_CAP_DR1, "DRAW_HORIZ_DR1" },
         { AV_CODEC_CAP_DELAY, "DELAY" },
         { AV_CODEC_CAP_SMALL_LAST_FRAME, "SMALL_LAST_FRAME" },
+#ifdef AV_CODEC_CAP_SUBFRAMES
         { AV_CODEC_CAP_SUBFRAMES, "SUBFRAMES" },
+#endif
         { AV_CODEC_CAP_EXPERIMENTAL, "EXPERIMENTAL" },
         { AV_CODEC_CAP_CHANNEL_CONF, "CHANNEL_CONF" },
         { AV_CODEC_CAP_FRAME_THREADS, "FRAME_THREADS" },
