@@ -15,7 +15,7 @@
 // We mean it.
 //
 
-#include "qffmpeghwaccel_p.h"
+#include <QtFFmpegMediaPluginImpl/private/qffmpeghwaccel_p.h>
 #include <memory>
 
 namespace QFFmpeg {
