@@ -5,15 +5,16 @@
 #include "qffmpegmediaformatinfo_p.h"
 #include <qloggingcategory.h>
 
-static Q_LOGGING_CATEGORY(qLcResampler, "qt.multimedia.ffmpeg.resampler")
-static Q_LOGGING_CATEGORY(qLcResamplerTrace, "qt.multimedia.ffmpeg.resampler.trace")
+Q_STATIC_LOGGING_CATEGORY(qLcResampler, "qt.multimedia.ffmpeg.resampler")
+Q_STATIC_LOGGING_CATEGORY(qLcResamplerTrace, "qt.multimedia.ffmpeg.resampler.trace")
 
 QT_BEGIN_NAMESPACE
 
 using namespace QFFmpeg;
 
-QFFmpegResampler::QFFmpegResampler(const QAudioFormat &inputFormat, const QAudioFormat &outputFormat) :
-    m_inputFormat(inputFormat), m_outputFormat(outputFormat)
+QFFmpegResampler::QFFmpegResampler(const QAudioFormat &inputFormat,
+                                   const QAudioFormat &outputFormat, qint64 startTime)
+    : m_inputFormat(inputFormat), m_outputFormat(outputFormat), m_startTime(startTime)
 {
     Q_ASSERT(inputFormat.isValid());
     Q_ASSERT(outputFormat.isValid());
@@ -49,6 +50,36 @@ QFFmpegResampler::QFFmpegResampler(const CodecContext *codecContext,
                                     << "us. From: " << inputAvFormat << " to: " << outputAvFormat;
 }
 
+template <typename... Args>
+std::unique_ptr<QFFmpegResampler> QFFmpegResampler::createImpl(Args... args)
+{
+    std::unique_ptr<QFFmpegResampler> resampler{
+        new QFFmpegResampler(std::forward<Args>(args)...),
+    };
+    if (resampler->isInitialized())
+        return resampler;
+    return nullptr;
+}
+
+std::unique_ptr<QFFmpegResampler>
+QFFmpegResampler::createFromInputFormat(const QAudioFormat &inputFormat,
+                                        const QAudioFormat &outputFormat, qint64 startTime)
+{
+    return createImpl(inputFormat, outputFormat, startTime);
+}
+
+std::unique_ptr<QFFmpegResampler>
+QFFmpegResampler::createFromCodecContext(const CodecContext *codecContext,
+                                         const QAudioFormat &outputFormat, qint64 startTime)
+{
+    return createImpl(codecContext, outputFormat, startTime);
+}
+
+bool QFFmpegResampler::isInitialized() const
+{
+    return m_resampler != nullptr;
+}
+
 QFFmpegResampler::~QFFmpegResampler() = default;
 
 QAudioBuffer QFFmpegResampler::resample(const char* data, size_t size)
