@@ -103,7 +103,7 @@ QGStreamerPlatformSpecificInterfaceImplementation::gstPipeline(QMediaCaptureSess
     return gstreamerCapture ? gstreamerCapture->pipeline().pipeline() : nullptr;
 }
 
-Q_LOGGING_CATEGORY(lcGstreamer, "qt.multimedia.gstreamer")
+Q_STATIC_LOGGING_CATEGORY(lcGstreamer, "qt.multimedia.gstreamer")
 
 namespace {
 
@@ -145,12 +145,13 @@ QGstreamerIntegration::QGstreamerIntegration()
     : QPlatformMediaIntegration(QLatin1String("gstreamer"))
 {
     gst_init(nullptr, nullptr);
-    qCDebug(lcGstreamer) << "Using gstreamer version: " << gst_version_string();
+
+    const QGString version{ gst_version_string() };
+    qCInfo(lcGstreamer) << "Using Qt multimedia with GStreamer version:" << version.asStringView();
 
     GstRegistry *reg = gst_registry_get();
 
     if constexpr (!GST_CHECK_VERSION(1, 22, 0)) {
-        GstRegistry* reg = gst_registry_get();
         for (const char *name : vaapiPluginNames)
             rankDownPlugin(reg, name);
     }
@@ -169,6 +170,15 @@ QGstreamerIntegration::QGstreamerIntegration()
     qGstRegisterQIODeviceHandler(nullptr);
 }
 
+QGstreamerIntegration::~QGstreamerIntegration()
+{
+    // by default we don't deinit, as the application may have initialized gstreamer
+    // (gst_init/deinit is not refcounted).
+    // however it's useful to force deinitialization for leak detection in qt's unit tests.
+    if (qEnvironmentVariableIsSet("QT_GSTREAMER_DEINIT"))
+        gst_deinit();
+}
+
 QPlatformMediaFormatInfo *QGstreamerIntegration::createFormatInfo()
 {
     return new QGstreamerFormatInfo();
