@@ -9,11 +9,12 @@
 #include <qaudiodevice.h>
 #include <private/qmemoryvideobuffer_p.h>
 #include <private/qvideoframe_p.h>
-#include <private/qwindowsmfdefs_p.h>
 #include <QtCore/private/qcomptr_p.h>
 #include <QtCore/qdebug.h>
 
 #include <mmdeviceapi.h>
+#include <mfidl.h>
+#include <cguid.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -45,8 +46,8 @@ HRESULT QWindowsMediaDeviceReader::createSource(const QString &deviceId, bool vi
     if (SUCCEEDED(hr)) {
 
         hr = sourceAttributes->SetGUID(MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE,
-                                       video ? QMM_MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_GUID
-                                             : QMM_MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_AUDCAP_GUID);
+                                       video ? MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_GUID
+                                             : MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_AUDCAP_GUID);
         if (SUCCEEDED(hr)) {
 
             hr = sourceAttributes->SetString(video ? MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_SYMBOLIC_LINK
@@ -386,7 +387,9 @@ HRESULT QWindowsMediaDeviceReader::startMonitoring()
 
                                                 IMFSimpleAudioVolume *audioVolume = nullptr;
 
-                                                if (SUCCEEDED(MFGetService(mediaSink, QMM_MR_POLICY_VOLUME_SERVICE, IID_PPV_ARGS(&audioVolume)))) {
+                                                if (SUCCEEDED(MFGetService(
+                                                            mediaSink, MR_POLICY_VOLUME_SERVICE,
+                                                            IID_PPV_ARGS(&audioVolume)))) {
                                                     audioVolume->SetMasterVolume(float(m_outputVolume));
                                                     audioVolume->SetMute(m_outputMuted);
                                                     audioVolume->Release();
@@ -651,7 +654,7 @@ QMediaRecorder::Error QWindowsMediaDeviceReader::startRecording(
     if (FAILED(hr))
         return QMediaRecorder::ResourceError;
 
-    hr = writerAttributes->SetGUID(QMM_MF_TRANSCODE_CONTAINERTYPE, container);
+    hr = writerAttributes->SetGUID(MF_TRANSCODE_CONTAINERTYPE, container);
     if (FAILED(hr))
         return QMediaRecorder::ResourceError;
 
@@ -829,7 +832,7 @@ void QWindowsMediaDeviceReader::setOutputMuted(bool muted)
 
     if (m_active && m_monitorSink) {
         IMFSimpleAudioVolume *audioVolume = nullptr;
-        if (SUCCEEDED(MFGetService(m_monitorSink, QMM_MR_POLICY_VOLUME_SERVICE,
+        if (SUCCEEDED(MFGetService(m_monitorSink, MR_POLICY_VOLUME_SERVICE,
                                    IID_PPV_ARGS(&audioVolume)))) {
             audioVolume->SetMute(m_outputMuted);
             audioVolume->Release();
@@ -845,7 +848,7 @@ void QWindowsMediaDeviceReader::setOutputVolume(qreal volume)
 
     if (m_active && m_monitorSink) {
         IMFSimpleAudioVolume *audioVolume = nullptr;
-        if (SUCCEEDED(MFGetService(m_monitorSink, QMM_MR_POLICY_VOLUME_SERVICE,
+        if (SUCCEEDED(MFGetService(m_monitorSink, MR_POLICY_VOLUME_SERVICE,
                                    IID_PPV_ARGS(&audioVolume)))) {
             audioVolume->SetMasterVolume(float(m_outputVolume));
             audioVolume->Release();
