@@ -15,16 +15,18 @@
 // We mean it.
 //
 
-#include <private/qplatformmediaintegration_p.h>
-#include "qwindowsvideodevices_p.h"
+#include <QtMultimedia/private/qcominitializer_p.h>
+#include <QtMultimedia/private/qplatformmediaintegration_p.h>
+#include <QtMultimedia/private/qwindowsvideodevices_p.h>
 
 QT_BEGIN_NAMESPACE
 
-class QWindowsMediaDevices;
+class QWindowsAudioDevices;
 class QWindowsFormatInfo;
 
 class QWindowsMediaIntegration : public QPlatformMediaIntegration
 {
+    Q_OBJECT
 public:
     QWindowsMediaIntegration();
     ~QWindowsMediaIntegration();
@@ -43,6 +45,9 @@ protected:
     QPlatformMediaFormatInfo *createFormatInfo() override;
 
     QPlatformVideoDevices *createVideoDevices() override;
+
+private:
+    QComInitializer m_comInitializer;
 };
 
 QT_END_NAMESPACE
