@@ -3,15 +3,14 @@
 
 
 #include <QtTest/QtTest>
-#include <QtCore/qlocale.h>
-#include <qaudiodevice.h>
-
-#include <QStringList>
-#include <QList>
-#include <QMediaDevices>
 
+#include <QtCore/qstringlist.h>
+#include <QtCore/qlist.h>
+#include <QtMultimedia/qmediadevices.h>
+#include <QtMultimedia/qaudiodevice.h>
 
 #include <private/mediabackendutils_p.h>
+#include <private/osdetection_p.h>
 
 class tst_QAudioDeviceBackend : public QObject
 {
@@ -90,10 +89,8 @@ void tst_QAudioDeviceBackend::sampleRates()
 
 void tst_QAudioDeviceBackend::isFormatSupported()
 {
-#ifdef Q_OS_MACOS
-    if (isCI())
+    if (isCI() && isMacOS)
         QSKIP("MacOS vms don't seem to support 44100hz on CI?");
-#endif
 
     QAudioFormat format;
     format.setSampleRate(44100);
@@ -106,9 +103,17 @@ void tst_QAudioDeviceBackend::isFormatSupported()
 
 void tst_QAudioDeviceBackend::preferred()
 {
-    QAudioFormat format = device->preferredFormat();
-    QVERIFY(format.isValid());
-    QVERIFY(device->isFormatSupported(format));
+    auto isPreferredSupported = [](const QAudioDevice &dev) {
+        QAudioFormat format = dev.preferredFormat();
+        QVERIFY(format.isValid());
+        QVERIFY(dev.isFormatSupported(format));
+    };
+
+    for (const auto &dev: QMediaDevices::audioOutputs())
+        isPreferredSupported(dev);
+
+    for (const auto &dev: QMediaDevices::audioInputs())
+        isPreferredSupported(dev);
 }
 
 // QAudioDevice's assignOperator method
