@@ -1,7 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-
-//TESTED_COMPONENT=src/multimedia
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 #include <QtGui/QImageReader>
@@ -9,6 +7,7 @@
 #include <QDebug>
 #include <QVideoSink>
 #include <QVideoWidget>
+#include <QSysInfo>
 
 #include <qcamera.h>
 #include <qcameradevice.h>
@@ -22,13 +21,21 @@
 #include <qaudiodevice.h>
 #include <qaudiodecoder.h>
 #include <qaudiobuffer.h>
+#include <qscreencapture.h>
+#include <qwindowcapture.h>
+#include <qaudiobufferinput.h>
+#include <qvideoframeinput.h>
 
 #include <qcamera.h>
 #include <QMediaFormat>
 #include <QtMultimediaWidgets/QVideoWidget>
 
+#include <private/mediabackendutils_p.h>
+
 QT_USE_NAMESPACE
 
+// NOLINTBEGIN(readability-convert-member-functions-to-static)
+
 /*
  This is the backend conformance test.
 
@@ -44,6 +51,12 @@ private slots:
 
     void testAudioMute();
     void stress_test_setup_and_teardown();
+    void stress_test_setup_and_teardown_keep_session();
+    void stress_test_setup_and_teardown_keep_recorder();
+    void stress_test_setup_and_teardown_keep_camera();
+    void stress_test_setup_and_teardown_keep_audioinput();
+    void stress_test_setup_and_teardown_keep_audiooutput();
+    void stress_test_setup_and_teardown_keep_video();
 
     void record_video_without_preview();
 
@@ -52,9 +65,16 @@ private slots:
     void can_change_AudioInput_during_recording();
     void disconnects_deleted_AudioInput();
     void can_move_AudioInput_between_sessions();
+
     void disconnects_deleted_AudioOutput();
     void can_move_AudioOutput_between_sessions_and_player();
 
+    void disconnects_deleted_AudioBufferInput();
+    void can_move_AudioBufferInput_between_sessions();
+
+    void disconnects_deleted_VideoFrameInput();
+    void can_move_VideoFrameInput_between_sessions();
+
     void can_add_and_remove_Camera();
     void can_move_Camera_between_sessions();
     void can_disconnect_Camera_when_recording();
@@ -76,6 +96,12 @@ private slots:
     void capture_is_not_available_when_Camera_is_null();
     void can_add_ImageCapture_and_capture_during_recording();
 
+    void can_switch_audio_output();
+    void can_switch_audio_input();
+
+    void can_clear_audio_output();
+    void can_clear_audio_input();
+
 private:
     void recordOk(QMediaCaptureSession &session);
     void recordFail(QMediaCaptureSession &session);
@@ -87,8 +113,8 @@ void tst_QMediaCaptureSession::recordOk(QMediaCaptureSession &session)
     QMediaRecorder recorder;
     session.setRecorder(&recorder);
 
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     recorder.record();
     QTRY_VERIFY_WITH_TIMEOUT(recorder.recorderState() == QMediaRecorder::RecordingState, 2000);
@@ -107,7 +133,7 @@ void tst_QMediaCaptureSession::recordOk(QMediaCaptureSession &session)
 void tst_QMediaCaptureSession::recordFail(QMediaCaptureSession &session)
 {
     QMediaRecorder recorder;
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
 
     session.setRecorder(&recorder);
     recorder.record();
@@ -118,6 +144,8 @@ void tst_QMediaCaptureSession::recordFail(QMediaCaptureSession &session)
 
 void tst_QMediaCaptureSession::stress_test_setup_and_teardown()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
     for (int i = 0; i < 50; i++) {
         QMediaCaptureSession session;
         QMediaRecorder recorder;
@@ -126,8 +154,160 @@ void tst_QMediaCaptureSession::stress_test_setup_and_teardown()
         QAudioOutput output;
         QVideoWidget video;
 
-        session.setAudioInput(&input);
-        session.setAudioOutput(&output);
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
+        session.setRecorder(&recorder);
+        session.setCamera(&camera);
+        session.setVideoOutput(&video);
+
+        QRandomGenerator rng;
+        QTest::qWait(rng.bounded(200));
+    }
+}
+
+void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_session()
+{
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
+    QMediaCaptureSession session;
+    for (int i = 0; i < 50; i++) {
+        QMediaRecorder recorder;
+        QCamera camera;
+        QAudioInput input;
+        QAudioOutput output;
+        QVideoWidget video;
+
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
+        session.setRecorder(&recorder);
+        session.setCamera(&camera);
+        session.setVideoOutput(&video);
+
+        QRandomGenerator rng;
+        QTest::qWait(rng.bounded(200));
+    }
+}
+
+void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_recorder()
+{
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
+    QMediaCaptureSession session;
+    QMediaRecorder recorder;
+    for (int i = 0; i < 50; i++) {
+        QCamera camera;
+        QAudioInput input;
+        QAudioOutput output;
+        QVideoWidget video;
+
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
+        session.setRecorder(&recorder);
+        session.setCamera(&camera);
+        session.setVideoOutput(&video);
+
+        QRandomGenerator rng;
+        QTest::qWait(rng.bounded(200));
+    }
+}
+
+void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_camera()
+{
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
+    QCamera camera;
+    for (int i = 0; i < 50; i++) {
+        QMediaCaptureSession session;
+        QMediaRecorder recorder;
+        QAudioInput input;
+        QAudioOutput output;
+        QVideoWidget video;
+
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
+        session.setRecorder(&recorder);
+        session.setCamera(&camera);
+        session.setVideoOutput(&video);
+
+        QRandomGenerator rng;
+        QTest::qWait(rng.bounded(200));
+    }
+}
+
+void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audioinput()
+{
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
+    QAudioInput input;
+    for (int i = 0; i < 50; i++) {
+        QMediaCaptureSession session;
+        QMediaRecorder recorder;
+        QCamera camera;
+        QAudioOutput output;
+        QVideoWidget video;
+
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
+        session.setRecorder(&recorder);
+        session.setCamera(&camera);
+        session.setVideoOutput(&video);
+
+        QRandomGenerator rng;
+        QTest::qWait(rng.bounded(200));
+    }
+}
+
+void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_audiooutput()
+{
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
+    QAudioOutput output;
+    for (int i = 0; i < 50; i++) {
+        QMediaCaptureSession session;
+        QMediaRecorder recorder;
+        QCamera camera;
+        QAudioInput input;
+        QVideoWidget video;
+
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
+        session.setRecorder(&recorder);
+        session.setCamera(&camera);
+        session.setVideoOutput(&video);
+
+        QRandomGenerator rng;
+        QTest::qWait(rng.bounded(200));
+    }
+}
+
+void tst_QMediaCaptureSession::stress_test_setup_and_teardown_keep_video()
+{
+    QSKIP_GSTREAMER("QTBUG-124005: spurious seek failures on CI");
+
+    QVideoWidget video;
+    for (int i = 0; i < 50; i++) {
+        QMediaCaptureSession session;
+        QMediaRecorder recorder;
+        QCamera camera;
+        QAudioInput input;
+        QAudioOutput output;
+
+        if (!input.device().isNull())
+            session.setAudioInput(&input);
+        if (!output.device().isNull())
+            session.setAudioOutput(&output);
         session.setRecorder(&recorder);
         session.setCamera(&camera);
         session.setVideoOutput(&video);
@@ -149,11 +329,12 @@ void tst_QMediaCaptureSession::record_video_without_preview()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     camera.setActive(true);
     QTRY_COMPARE(cameraChanged.size(), 1);
+    QTRY_COMPARE(camera.isActive(), true);
 
     recordOk(session);
     QVERIFY(!QTest::currentTestFailed());
@@ -173,8 +354,8 @@ void tst_QMediaCaptureSession::can_add_and_remove_AudioInput_with_and_without_Au
         QSKIP("No audio input available");
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy audioOutputChanged(&session, SIGNAL(audioOutputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy audioOutputChanged(&session, &QMediaCaptureSession::audioOutputChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -205,10 +386,10 @@ void tst_QMediaCaptureSession::can_change_AudioDevices_on_attached_AudioInput()
         QSKIP("Two audio inputs are not available");
 
     QAudioInput input(audioInputs[0]);
-    QSignalSpy deviceChanged(&input, SIGNAL(deviceChanged()));
+    QSignalSpy deviceChanged(&input, &QAudioInput::deviceChanged);
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -240,9 +421,9 @@ void tst_QMediaCaptureSession::can_change_AudioInput_during_recording()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -274,7 +455,7 @@ void tst_QMediaCaptureSession::disconnects_deleted_AudioInput()
         QSKIP("No audio input available");
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
     {
         QAudioInput input;
         session.setAudioInput(&input);
@@ -291,13 +472,13 @@ void tst_QMediaCaptureSession::can_move_AudioInput_between_sessions()
 
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy audioInputChanged0(&session0, SIGNAL(audioInputChanged()));
-    QSignalSpy audioInputChanged1(&session1, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged0(&session0, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy audioInputChanged1(&session1, &QMediaCaptureSession::audioInputChanged);
 
     QAudioInput input;
     {
         QMediaCaptureSession session2;
-        QSignalSpy audioInputChanged2(&session2, SIGNAL(audioInputChanged()));
+        QSignalSpy audioInputChanged2(&session2, &QMediaCaptureSession::audioInputChanged);
         session2.setAudioInput(&input);
         QTRY_COMPARE(audioInputChanged2.size(), 1);
     }
@@ -318,7 +499,7 @@ void tst_QMediaCaptureSession::disconnects_deleted_AudioOutput()
         QSKIP("No audio output available");
 
     QMediaCaptureSession session;
-    QSignalSpy audioOutputChanged(&session, SIGNAL(audioOutputChanged()));
+    QSignalSpy audioOutputChanged(&session, &QMediaCaptureSession::audioOutputChanged);
     {
         QAudioOutput output;
         session.setAudioOutput(&output);
@@ -333,17 +514,18 @@ void tst_QMediaCaptureSession::can_move_AudioOutput_between_sessions_and_player(
     if (QMediaDevices::audioOutputs().isEmpty())
         QSKIP("No audio output available");
 
+    QAudioOutput output;
+
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
     QMediaPlayer player;
-    QSignalSpy audioOutputChanged0(&session0, SIGNAL(audioOutputChanged()));
-    QSignalSpy audioOutputChanged1(&session1, SIGNAL(audioOutputChanged()));
-    QSignalSpy audioOutputChangedPlayer(&player, SIGNAL(audioOutputChanged()));
+    QSignalSpy audioOutputChanged0(&session0, &QMediaCaptureSession::audioOutputChanged);
+    QSignalSpy audioOutputChanged1(&session1, &QMediaCaptureSession::audioOutputChanged);
+    QSignalSpy audioOutputChangedPlayer(&player, &QMediaPlayer::audioOutputChanged);
 
-    QAudioOutput output;
     {
         QMediaCaptureSession session2;
-        QSignalSpy audioOutputChanged2(&session2, SIGNAL(audioOutputChanged()));
+        QSignalSpy audioOutputChanged2(&session2, &QMediaCaptureSession::audioOutputChanged);
         session2.setAudioOutput(&output);
         QTRY_COMPARE(audioOutputChanged2.size(), 1);
     }
@@ -373,6 +555,89 @@ void tst_QMediaCaptureSession::can_move_AudioOutput_between_sessions_and_player(
     QVERIFY(player.audioOutput() == nullptr);
 }
 
+void tst_QMediaCaptureSession::disconnects_deleted_AudioBufferInput()
+{
+    QMediaCaptureSession session;
+    QSignalSpy audioBufferInputChanged(&session, &QMediaCaptureSession::audioBufferInputChanged);
+    {
+        QAudioBufferInput input;
+        session.setAudioBufferInput(&input);
+        QTRY_COMPARE(audioBufferInputChanged.size(), 1);
+    }
+    QCOMPARE(session.audioBufferInput(), nullptr);
+    QCOMPARE(audioBufferInputChanged.size(), 2);
+}
+
+void tst_QMediaCaptureSession::can_move_AudioBufferInput_between_sessions()
+{
+    QMediaCaptureSession session0;
+    QMediaCaptureSession session1;
+    QSignalSpy audioBufferInputChanged0(&session0, &QMediaCaptureSession::audioBufferInputChanged);
+    QSignalSpy audioBufferInputChanged1(&session1, &QMediaCaptureSession::audioBufferInputChanged);
+
+    QAudioBufferInput input;
+    {
+        QMediaCaptureSession session2;
+        QSignalSpy audioBufferInputChanged2(&session2,
+                                            &QMediaCaptureSession::audioBufferInputChanged);
+        session2.setAudioBufferInput(&input);
+        QCOMPARE(audioBufferInputChanged2.size(), 1);
+    }
+    session0.setAudioBufferInput(&input);
+    QCOMPARE(audioBufferInputChanged0.size(), 1);
+    QCOMPARE(session0.audioBufferInput(), &input);
+    QCOMPARE(input.captureSession(), &session0);
+
+    session1.setAudioBufferInput(&input);
+
+    QCOMPARE(audioBufferInputChanged0.size(), 2);
+    QCOMPARE(session0.audioBufferInput(), nullptr);
+    QCOMPARE(audioBufferInputChanged1.size(), 1);
+    QCOMPARE(session1.audioBufferInput(), &input);
+    QCOMPARE(input.captureSession(), &session1);
+}
+
+void tst_QMediaCaptureSession::disconnects_deleted_VideoFrameInput()
+{
+    QMediaCaptureSession session;
+    QSignalSpy videoFrameInputChanged(&session, &QMediaCaptureSession::videoFrameInputChanged);
+    {
+        QVideoFrameInput input;
+        session.setVideoFrameInput(&input);
+        QTRY_COMPARE(videoFrameInputChanged.size(), 1);
+    }
+    QCOMPARE(session.videoFrameInput(), nullptr);
+    QCOMPARE(videoFrameInputChanged.size(), 2);
+}
+
+void tst_QMediaCaptureSession::can_move_VideoFrameInput_between_sessions()
+{
+    QMediaCaptureSession session0;
+    QMediaCaptureSession session1;
+    QSignalSpy videoFrameInputChanged0(&session0, &QMediaCaptureSession::videoFrameInputChanged);
+    QSignalSpy videoFrameInputChanged1(&session1, &QMediaCaptureSession::videoFrameInputChanged);
+
+    QVideoFrameInput input;
+    {
+        QMediaCaptureSession session2;
+        QSignalSpy videoFrameInputChanged2(&session2,
+                                           &QMediaCaptureSession::videoFrameInputChanged);
+        session2.setVideoFrameInput(&input);
+        QCOMPARE(videoFrameInputChanged2.size(), 1);
+    }
+    session0.setVideoFrameInput(&input);
+    QCOMPARE(videoFrameInputChanged0.size(), 1);
+    QCOMPARE(session0.videoFrameInput(), &input);
+    QCOMPARE(input.captureSession(), &session0);
+
+    session1.setVideoFrameInput(&input);
+
+    QCOMPARE(videoFrameInputChanged0.size(), 2);
+    QCOMPARE(session0.videoFrameInput(), nullptr);
+    QCOMPARE(videoFrameInputChanged1.size(), 1);
+    QCOMPARE(session1.videoFrameInput(), &input);
+    QCOMPARE(input.captureSession(), &session1);
+}
 
 void tst_QMediaCaptureSession::can_add_and_remove_Camera()
 {
@@ -386,11 +651,12 @@ void tst_QMediaCaptureSession::can_add_and_remove_Camera()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     camera.setActive(true);
     QTRY_COMPARE(cameraChanged.size(), 1);
+    QTRY_COMPARE(camera.isActive(), true);
 
     session.setCamera(nullptr);
     QTRY_COMPARE(cameraChanged.size(), 2);
@@ -406,13 +672,13 @@ void tst_QMediaCaptureSession::can_move_Camera_between_sessions()
 {
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy cameraChanged0(&session0, SIGNAL(cameraChanged()));
-    QSignalSpy cameraChanged1(&session1, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged0(&session0, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy cameraChanged1(&session1, &QMediaCaptureSession::cameraChanged);
     {
         QCamera camera;
         {
             QMediaCaptureSession session2;
-            QSignalSpy cameraChanged2(&session2, SIGNAL(cameraChanged()));
+            QSignalSpy cameraChanged2(&session2, &QMediaCaptureSession::cameraChanged);
             session2.setCamera(&camera);
             QTRY_COMPARE(cameraChanged2.size(), 1);
         }
@@ -446,13 +712,14 @@ void tst_QMediaCaptureSession::can_disconnect_Camera_when_recording()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     camera.setActive(true);
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
+    QTRY_COMPARE(camera.isActive(), true);
 
     durationChanged.clear();
     recorder.record();
@@ -487,11 +754,12 @@ void tst_QMediaCaptureSession::can_add_and_remove_different_Cameras()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     camera.setActive(true);
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
+    QTRY_COMPARE(camera.isActive(), true);
 
     session.setCamera(nullptr);
     QTRY_COMPARE(cameraChanged.size(), 2);
@@ -499,6 +767,7 @@ void tst_QMediaCaptureSession::can_add_and_remove_different_Cameras()
     session.setCamera(&camera2);
     camera2.setActive(true);
     QTRY_COMPARE(cameraChanged.size(), 3);
+    QTRY_COMPARE(camera2.isActive(), true);
 
     recordOk(session);
     QVERIFY(!QTest::currentTestFailed());
@@ -518,8 +787,8 @@ void tst_QMediaCaptureSession::can_change_CameraDevice_on_attached_Camera()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraDeviceChanged(&camera, SIGNAL(cameraDeviceChanged()));
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraDeviceChanged(&camera, &QCamera::cameraDeviceChanged);
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
@@ -528,12 +797,15 @@ void tst_QMediaCaptureSession::can_change_CameraDevice_on_attached_Camera()
     QVERIFY(!QTest::currentTestFailed());
 
     camera.setActive(true);
+    QTRY_COMPARE(camera.isActive(), true);
+
     recordOk(session);
     QVERIFY(!QTest::currentTestFailed());
 
     camera.setCameraDevice(cameraDevices[1]);
     camera.setActive(true);
     QTRY_COMPARE(cameraDeviceChanged.size(), 1);
+    QTRY_COMPARE(camera.isActive(), true);
 
     recordOk(session);
     QVERIFY(!QTest::currentTestFailed());
@@ -552,8 +824,8 @@ void tst_QMediaCaptureSession::can_change_VideoOutput_with_and_without_camera()
 
     QMediaCaptureSession session;
 
-    QSignalSpy videoOutputChanged(&session, SIGNAL(videoOutputChanged()));
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy videoOutputChanged(&session, &QMediaCaptureSession::videoOutputChanged);
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
@@ -588,14 +860,15 @@ void tst_QMediaCaptureSession::can_change_VideoOutput_when_recording()
 
     session.setRecorder(&recorder);
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
-    QSignalSpy videoOutputChanged(&session, SIGNAL(videoOutputChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
+    QSignalSpy videoOutputChanged(&session, &QMediaCaptureSession::videoOutputChanged);
 
     camera.setActive(true);
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
+    QTRY_COMPARE(camera.isActive(), true);
 
     recorder.record();
     QTRY_VERIFY(recorder.recorderState() == QMediaRecorder::RecordingState);
@@ -630,8 +903,8 @@ void tst_QMediaCaptureSession::can_add_and_remove_recorders()
     QMediaRecorder recorder2;
     QMediaCaptureSession session;
 
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy recorderChanged(&session, SIGNAL(recorderChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy recorderChanged(&session, &QMediaCaptureSession::recorderChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -653,13 +926,13 @@ void tst_QMediaCaptureSession::can_move_Recorder_between_sessions()
 {
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy recorderChanged0(&session0, SIGNAL(recorderChanged()));
-    QSignalSpy recorderChanged1(&session1, SIGNAL(recorderChanged()));
+    QSignalSpy recorderChanged0(&session0, &QMediaCaptureSession::recorderChanged);
+    QSignalSpy recorderChanged1(&session1, &QMediaCaptureSession::recorderChanged);
     {
         QMediaRecorder recorder;
         {
             QMediaCaptureSession session2;
-            QSignalSpy recorderChanged2(&session2, SIGNAL(recorderChanged()));
+            QSignalSpy recorderChanged2(&session2, &QMediaCaptureSession::recorderChanged);
             session2.setRecorder(&recorder);
             QTRY_COMPARE(recorderChanged2.size(), 1);
         }
@@ -696,7 +969,7 @@ void tst_QMediaCaptureSession::can_record_AudioInput_with_null_AudioDevice()
     QAudioInput input(nullDevice);
 
     QMediaCaptureSession session;
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -714,12 +987,14 @@ void tst_QMediaCaptureSession::can_record_Camera_with_null_CameraDevice()
     QCamera camera(nullDevice);
 
     QMediaCaptureSession session;
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
 
     session.setCamera(&camera);
     QTRY_COMPARE(cameraChanged.size(), 1);
 
     camera.setActive(true);
+    QTRY_COMPARE(camera.isActive(), true);
+
     recordOk(session);
     QVERIFY(!QTest::currentTestFailed());
 }
@@ -730,13 +1005,15 @@ void tst_QMediaCaptureSession::recording_stops_when_recorder_removed()
     if (input.device().isNull())
         QSKIP("Recording source not available");
 
+    QSKIP_GSTREAMER("spurious failures on gstreamer");
+
     QMediaRecorder recorder;
     QMediaCaptureSession session;
 
-    QSignalSpy audioInputChanged(&session, SIGNAL(audioInputChanged()));
-    QSignalSpy recorderChanged(&session, SIGNAL(recorderChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy audioInputChanged(&session, &QMediaCaptureSession::audioInputChanged);
+    QSignalSpy recorderChanged(&session, &QMediaCaptureSession::recorderChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     session.setAudioInput(&input);
     QTRY_COMPARE(audioInputChanged.size(), 1);
@@ -770,9 +1047,9 @@ void tst_QMediaCaptureSession::can_add_and_remove_ImageCapture()
     QImageCapture capture;
     QMediaCaptureSession session;
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy imageCaptureChanged(&session, SIGNAL(imageCaptureChanged()));
-    QSignalSpy readyForCaptureChanged(&capture, SIGNAL(readyForCaptureChanged(bool)));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy imageCaptureChanged(&session, &QMediaCaptureSession::imageCaptureChanged);
+    QSignalSpy readyForCaptureChanged(&capture, &QImageCapture::readyForCaptureChanged);
 
     QVERIFY(!capture.isAvailable());
     QVERIFY(!capture.isReadyForCapture());
@@ -789,6 +1066,8 @@ void tst_QMediaCaptureSession::can_add_and_remove_ImageCapture()
     QVERIFY(!capture.isReadyForCapture());
 
     camera.setActive(true);
+    QTRY_COMPARE(camera.isActive(), true);
+
     QTRY_COMPARE(readyForCaptureChanged.size(), 1);
     QVERIFY(capture.isReadyForCapture());
 
@@ -808,15 +1087,17 @@ void tst_QMediaCaptureSession::can_add_and_remove_ImageCapture()
 
 void tst_QMediaCaptureSession::can_move_ImageCapture_between_sessions()
 {
+    QSKIP_GSTREAMER("QTBUG-124005: Spurious failure on CI");
+
     QMediaCaptureSession session0;
     QMediaCaptureSession session1;
-    QSignalSpy imageCaptureChanged0(&session0, SIGNAL(imageCaptureChanged()));
-    QSignalSpy imageCaptureChanged1(&session1, SIGNAL(imageCaptureChanged()));
+    QSignalSpy imageCaptureChanged0(&session0, &QMediaCaptureSession::imageCaptureChanged);
+    QSignalSpy imageCaptureChanged1(&session1, &QMediaCaptureSession::imageCaptureChanged);
     {
         QImageCapture imageCapture;
         {
             QMediaCaptureSession session2;
-            QSignalSpy imageCaptureChanged2(&session2, SIGNAL(imageCaptureChanged()));
+            QSignalSpy imageCaptureChanged2(&session2, &QMediaCaptureSession::imageCaptureChanged);
             session2.setImageCapture(&imageCapture);
             QTRY_COMPARE(imageCaptureChanged2.size(), 1);
         }
@@ -838,7 +1119,6 @@ void tst_QMediaCaptureSession::can_move_ImageCapture_between_sessions()
     QVERIFY(session1.imageCapture() == nullptr);
 }
 
-
 void tst_QMediaCaptureSession::capture_is_not_available_when_Camera_is_null()
 {
     QCamera camera;
@@ -846,16 +1126,19 @@ void tst_QMediaCaptureSession::capture_is_not_available_when_Camera_is_null()
     if (!camera.isAvailable())
         QSKIP("No video input available");
 
+    QSKIP_GSTREAMER("Spurious failures");
+
     QImageCapture capture;
     QMediaCaptureSession session;
 
-    QSignalSpy cameraChanged(&session, SIGNAL(cameraChanged()));
-    QSignalSpy capturedSignal(&capture, SIGNAL(imageCaptured(int,QImage)));
-    QSignalSpy readyForCaptureChanged(&capture, SIGNAL(readyForCaptureChanged(bool)));
+    QSignalSpy cameraChanged(&session, &QMediaCaptureSession::cameraChanged);
+    QSignalSpy capturedSignal(&capture, &QImageCapture::imageCaptured);
+    QSignalSpy readyForCaptureChanged(&capture, &QImageCapture::readyForCaptureChanged);
 
     session.setImageCapture(&capture);
     session.setCamera(&camera);
     camera.setActive(true);
+    QTRY_COMPARE(camera.isActive(), true);
 
     QTRY_COMPARE(readyForCaptureChanged.size(), 1);
     QVERIFY(capture.isReadyForCapture());
@@ -881,19 +1164,22 @@ void tst_QMediaCaptureSession::can_add_ImageCapture_and_capture_during_recording
     if (!camera.isAvailable())
         QSKIP("No video input available");
 
+    QSKIP_GSTREAMER("spurious failures with gstreamer");
+
     QImageCapture capture;
     QMediaCaptureSession session;
     QMediaRecorder recorder;
 
-    QSignalSpy recorderChanged(&session, SIGNAL(recorderChanged()));
-    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
-    QSignalSpy imageCaptureChanged(&session, SIGNAL(imageCaptureChanged()));
-    QSignalSpy readyForCaptureChanged(&capture, SIGNAL(readyForCaptureChanged(bool)));
-    QSignalSpy capturedSignal(&capture, SIGNAL(imageCaptured(int,QImage)));
+    QSignalSpy recorderChanged(&session, &QMediaCaptureSession::recorderChanged);
+    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
+    QSignalSpy imageCaptureChanged(&session, &QMediaCaptureSession::imageCaptureChanged);
+    QSignalSpy readyForCaptureChanged(&capture, &QImageCapture::readyForCaptureChanged);
+    QSignalSpy capturedSignal(&capture, &QImageCapture::imageCaptured);
 
     session.setCamera(&camera);
     camera.setActive(true);
+    QTRY_COMPARE(camera.isActive(), true);
 
     session.setRecorder(&recorder);
     QTRY_COMPARE(recorderChanged.size(), 1);
@@ -941,28 +1227,26 @@ void tst_QMediaCaptureSession::testAudioMute()
     recorder.setOutputLocation(QStringLiteral("test"));
 
     QSignalSpy spy(&audioInput, &QAudioInput::mutedChanged);
-    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
 
     QMediaFormat format;
-    format.setAudioCodec(QMediaFormat::AudioCodec::MP3);
+    format.setAudioCodec(QMediaFormat::AudioCodec::Wave);
     recorder.setMediaFormat(format);
 
-    recorder.record();
     audioInput.setMuted(true);
 
+    recorder.record();
+
     QCOMPARE(spy.size(), 1);
     QCOMPARE(spy.last()[0], true);
 
     QTRY_VERIFY_WITH_TIMEOUT(recorder.recorderState() == QMediaRecorder::RecordingState, 2000);
     QVERIFY(durationChanged.wait(2000));
 
-    audioInput.setMuted(false);
-
-    QCOMPARE(spy.size(), 2);
-    QCOMPARE(spy.last()[0], false);
-
     recorder.stop();
 
+    QTRY_COMPARE(recorder.recorderState(), QMediaRecorder::StoppedState);
+
     QString actualLocation = recorder.actualLocation().toLocalFile();
 
     QVERIFY2(!actualLocation.isEmpty(), "Recorder did not save a file");
@@ -991,6 +1275,93 @@ void tst_QMediaCaptureSession::testAudioMute()
     decoder.stop();
 
     QFile(actualLocation).remove();
+
+    audioInput.setMuted(false);
+
+    QCOMPARE(spy.size(), 2);
+    QCOMPARE(spy.last()[0], false);
+}
+
+void tst_QMediaCaptureSession::can_switch_audio_input()
+{
+    // prepare
+    QMediaCaptureSession session;
+    QAudioInput in1;
+    session.setAudioInput(&in1);
+    QVERIFY(session.audioInput() != nullptr);
+    QAudioOutput out;
+    session.setAudioOutput(&out);
+    QVERIFY(session.audioOutput() != nullptr);
+
+    // exercise
+    QAudioInput in2;
+    QSignalSpy changeSpy(&session, &QMediaCaptureSession::audioInputChanged);
+    session.setAudioInput(&in2);
+
+    // validate
+    QVERIFY(session.audioInput() != nullptr);
+    QCOMPARE(changeSpy.count(), 1);
+}
+
+void tst_QMediaCaptureSession::can_switch_audio_output()
+{
+    // prepare
+    QMediaCaptureSession session;
+    QAudioInput in;
+    session.setAudioInput(&in);
+    QVERIFY(session.audioInput() != nullptr);
+    QAudioOutput out1;
+    session.setAudioOutput(&out1);
+    QVERIFY(session.audioOutput() != nullptr);
+
+    // exercise
+    QSignalSpy changeSpy(&session, &QMediaCaptureSession::audioOutputChanged);
+    QAudioOutput out2;
+    session.setAudioOutput(&out2);
+
+    // validate
+    QVERIFY(session.audioOutput() != nullptr);
+    QCOMPARE(changeSpy.count(), 1);
+}
+
+void tst_QMediaCaptureSession::can_clear_audio_input()
+{
+    // prepare
+    QMediaCaptureSession session;
+    QAudioInput in1;
+    session.setAudioInput(&in1);
+    QVERIFY(session.audioInput() != nullptr);
+    QAudioOutput out;
+    session.setAudioOutput(&out);
+    QVERIFY(session.audioOutput() != nullptr);
+
+    // exercise
+    QSignalSpy changeSpy(&session, &QMediaCaptureSession::audioInputChanged);
+    session.setAudioInput(nullptr);
+
+    // validate
+    QVERIFY(session.audioInput() == nullptr);
+    QCOMPARE(changeSpy.count(), 1);
+}
+
+void tst_QMediaCaptureSession::can_clear_audio_output()
+{
+    // prepare
+    QMediaCaptureSession session;
+    QAudioInput in;
+    session.setAudioInput(&in);
+    QVERIFY(session.audioInput() != nullptr);
+    QAudioOutput out1;
+    session.setAudioOutput(&out1);
+    QVERIFY(session.audioOutput() != nullptr);
+
+    // exercise
+    QSignalSpy changeSpy(&session, &QMediaCaptureSession::audioOutputChanged);
+    session.setAudioOutput(nullptr);
+
+    // validate
+    QVERIFY(session.audioOutput() == nullptr);
+    QCOMPARE(changeSpy.count(), 1);
 }
 
 QTEST_MAIN(tst_QMediaCaptureSession)
