@@ -1,5 +1,5 @@
 // Copyright (C) 2023 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef WINDOW_CAPTURE_WIDGET_H
 #define WINDOW_CAPTURE_WIDGET_H
@@ -29,7 +29,7 @@ public slots:
     void togglePattern();
 
 protected:
-    void paintEvent(QPaintEvent * /*event*/) override;
+    void paintEvent(QPaintEvent *) override;
 
 private:
     void drawColoredSquares(QPainter &p);
