@@ -1,7 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-
-//TESTED_COMPONENT=src/multimedia
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 #include <QDebug>
@@ -16,12 +14,15 @@
 #include <qobject.h>
 #include <qmediadevices.h>
 
-#include "qmockintegration_p.h"
+#include "qmockintegration.h"
 #include "qmockmediacapturesession.h"
 #include "qmockcamera.h"
 
+#include <private/mediabackendutils_p.h>
+
 QT_USE_NAMESPACE
 
+Q_ENABLE_MOCK_MULTIMEDIA_PLUGIN
 
 class tst_QCamera: public QObject
 {
@@ -72,13 +73,14 @@ private slots:
     void testSignalIsoSensitivityChanged();
     void testSignalShutterSpeedChanged();
     void testSignalFlashReady();
-
-private:
-    QMockIntegration integration;
 };
 
 void tst_QCamera::initTestCase()
 {
+#ifdef Q_OS_MACOS
+    if (isCI())
+        QSKIP("Flakiness on macOS CI, to be investigated, QTBUG-111812");
+#endif
 }
 
 void tst_QCamera::init()
@@ -197,14 +199,14 @@ void tst_QCamera::testSimpleCameraCapture()
     QCOMPARE(imageCapture.error(), QImageCapture::NoError);
     QVERIFY(imageCapture.errorString().isEmpty());
 
-    QSignalSpy errorSignal(&imageCapture, SIGNAL(errorOccurred(int,QImageCapture::Error,QString)));
-    imageCapture.captureToFile(QString::fromLatin1("/dev/null"));
+    QSignalSpy errorSignal(&imageCapture, &QImageCapture::errorOccurred);
+    imageCapture.captureToFile(QStringLiteral("/dev/null"));
     QCOMPARE(errorSignal.size(), 1);
     QCOMPARE(imageCapture.error(), QImageCapture::NotReadyError);
     QVERIFY(!imageCapture.errorString().isEmpty());
 
     camera.start();
-    imageCapture.captureToFile(QString::fromLatin1("/dev/null"));
+    imageCapture.captureToFile(QStringLiteral("/dev/null"));
     QCOMPARE(errorSignal.size(), 1);
     QCOMPARE(imageCapture.error(), QImageCapture::NoError);
     QVERIFY(imageCapture.errorString().isEmpty());
@@ -220,10 +222,10 @@ void tst_QCamera::testCameraCapture()
 
     QVERIFY(!imageCapture.isReadyForCapture());
 
-    QSignalSpy capturedSignal(&imageCapture, SIGNAL(imageCaptured(int,QImage)));
-    QSignalSpy errorSignal(&imageCapture, SIGNAL(errorOccurred(int,QImageCapture::Error,QString)));
+    QSignalSpy capturedSignal(&imageCapture, &QImageCapture::imageCaptured);
+    QSignalSpy errorSignal(&imageCapture, &QImageCapture::errorOccurred);
 
-    imageCapture.captureToFile(QString::fromLatin1("/dev/null"));
+    imageCapture.captureToFile(QStringLiteral("/dev/null"));
     QCOMPARE(capturedSignal.size(), 0);
     QCOMPARE(errorSignal.size(), 1);
     QCOMPARE(imageCapture.error(), QImageCapture::NotReadyError);
@@ -234,7 +236,7 @@ void tst_QCamera::testCameraCapture()
     QVERIFY(imageCapture.isReadyForCapture());
     QCOMPARE(errorSignal.size(), 0);
 
-    imageCapture.captureToFile(QString::fromLatin1("/dev/null"));
+    imageCapture.captureToFile(QStringLiteral("/dev/null"));
 
     QTRY_COMPARE(capturedSignal.size(), 1);
     QCOMPARE(errorSignal.size(), 0);
@@ -249,11 +251,11 @@ void tst_QCamera::testCameraCaptureMetadata()
     session.setCamera(&camera);
     session.setImageCapture(&imageCapture);
 
-    QSignalSpy metadataSignal(&imageCapture, SIGNAL(imageMetadataAvailable(int,const QMediaMetaData&)));
-    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
+    QSignalSpy metadataSignal(&imageCapture, &QImageCapture::imageMetadataAvailable);
+    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
 
     camera.start();
-    int id = imageCapture.captureToFile(QString::fromLatin1("/dev/null"));
+    int id = imageCapture.captureToFile(QStringLiteral("/dev/null"));
 
     QTRY_COMPARE(savedSignal.size(), 1);
 
@@ -419,7 +421,7 @@ void tst_QCamera::testCameraEncodingProperyChange()
     session.setCamera(&camera);
     session.setImageCapture(&imageCapture);
 
-    QSignalSpy activeChangedSignal(&camera, SIGNAL(activeChanged(bool)));
+    QSignalSpy activeChangedSignal(&camera, &QCamera::activeChanged);
 
     camera.start();
     QCOMPARE(camera.isActive(), true);
@@ -600,14 +602,14 @@ void tst_QCamera::testErrorSignal()
     QMediaCaptureSession session;
     QCamera camera;
     session.setCamera(&camera);
-    auto *service = integration.lastCaptureService();
-    Q_ASSERT(service);
-    Q_ASSERT(service->mockCameraControl);
+    auto *service = QMockIntegration::instance()->lastCaptureService();
+    QTEST_ASSERT(service);
+    QTEST_ASSERT(service->mockCameraControl);
 
-    QSignalSpy spyError(&camera, SIGNAL(errorOccurred(QCamera::Error,const QString&)));
+    QSignalSpy spyError(&camera, &QCamera::errorOccurred);
 
     /* Set the QPlatformCamera error and verify if the signal is emitted correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("Camera Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError, QStringLiteral("Camera Error"));
 
     QVERIFY(spyError.size() == 1);
     QCamera::Error err = qvariant_cast<QCamera::Error >(spyError.at(0).at(0));
@@ -616,7 +618,8 @@ void tst_QCamera::testErrorSignal()
     spyError.clear();
 
     /* Set the QPlatformCamera error and verify if the signal is emitted correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("InvalidRequestError Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError,
+                                            QStringLiteral("InvalidRequestError Error"));
     QVERIFY(spyError.size() == 1);
     err = qvariant_cast<QCamera::Error >(spyError.at(0).at(0));
     QVERIFY(err == QCamera::CameraError);
@@ -624,7 +627,8 @@ void tst_QCamera::testErrorSignal()
     spyError.clear();
 
     /* Set the QPlatformCamera error and verify if the signal is emitted correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("NotSupportedFeatureError Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError,
+                                            QStringLiteral("NotSupportedFeatureError Error"));
     QVERIFY(spyError.size() == 1);
     err = qvariant_cast<QCamera::Error >(spyError.at(0).at(0));
     QVERIFY(err == QCamera::CameraError);
@@ -637,18 +641,20 @@ void tst_QCamera::testError()
     QMediaCaptureSession session;
     QCamera camera;
     session.setCamera(&camera);
-    auto *service = integration.lastCaptureService();
+    auto *service = QMockIntegration::instance()->lastCaptureService();
 
     /* Set the QPlatformCamera error and verify if it is set correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("Camera Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError, QStringLiteral("Camera Error"));
     QVERIFY(camera.error() == QCamera::CameraError);
 
     /* Set the QPlatformCamera error and verify if it is set correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("InvalidRequestError Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError,
+                                            QStringLiteral("InvalidRequestError Error"));
     QVERIFY(camera.error() == QCamera::CameraError);
 
     /* Set the QPlatformCamera error and verify if it is set correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("CameraError Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError,
+                                            QStringLiteral("CameraError Error"));
     QVERIFY(camera.error() == QCamera::CameraError);
 
 }
@@ -659,19 +665,21 @@ void tst_QCamera::testErrorString()
     QMediaCaptureSession session;
     QCamera camera;
     session.setCamera(&camera);
-    auto *service = integration.lastCaptureService();
+    auto *service = QMockIntegration::instance()->lastCaptureService();
 
     /* Set the QPlatformCamera error and verify if it is set correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("Camera Error"));
-    QVERIFY(camera.errorString() == QString("Camera Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError, QStringLiteral("Camera Error"));
+    QVERIFY(camera.errorString() == QStringLiteral("Camera Error"));
 
     /* Set the QPlatformCamera error and verify if it is set correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("InvalidRequestError Error"));
-    QVERIFY(camera.errorString() == QString("InvalidRequestError Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError,
+                                            QStringLiteral("InvalidRequestError Error"));
+    QVERIFY(camera.errorString() == QStringLiteral("InvalidRequestError Error"));
 
     /* Set the QPlatformCamera error and verify if it is set correctly in QCamera */
-    service->mockCameraControl->setError(QCamera::CameraError,QString("CameraError Error"));
-    QVERIFY(camera.errorString() == QString("CameraError Error"));
+    service->mockCameraControl->updateError(QCamera::CameraError,
+                                            QStringLiteral("CameraError Error"));
+    QVERIFY(camera.errorString() == QStringLiteral("CameraError Error"));
 }
 
 void tst_QCamera::testSetCameraFormat()
@@ -681,7 +689,7 @@ void tst_QCamera::testSetCameraFormat()
     auto videoFormats = device.videoFormats();
     QVERIFY(videoFormats.size());
     QCameraFormat cameraFormat = videoFormats.first();
-    QSignalSpy spy(&camera, SIGNAL(cameraFormatChanged()));
+    QSignalSpy spy(&camera, &QCamera::cameraFormatChanged);
     QVERIFY(spy.size() == 0);
     camera.setCameraFormat(cameraFormat);
     QCOMPARE(spy.size(), 1);
@@ -733,7 +741,7 @@ void tst_QCamera::testZoomChanged()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy spy(&camera, SIGNAL(zoomFactorChanged(float)));
+    QSignalSpy spy(&camera, &QCamera::zoomFactorChanged);
     QVERIFY(spy.size() == 0);
     camera.setZoomFactor(2.0);
     QVERIFY(spy.size() == 1);
@@ -748,10 +756,10 @@ void tst_QCamera::testMaxZoomChangedSignal()
     QMediaCaptureSession session;
     QCamera camera;
     session.setCamera(&camera);
-    QMockCamera *mock = integration.lastCamera();
+    QMockCamera *mock = QMockIntegration::instance()->lastCamera();
 
     // ### change max zoom factor on backend, e.g. by changing camera
-    QSignalSpy spy(&camera, SIGNAL(maximumZoomFactorChanged(float)));
+    QSignalSpy spy(&camera, &QCamera::maximumZoomFactorChanged);
     mock->maximumZoomFactorChanged(55);
     QVERIFY(spy.size() == 1);
     QCOMPARE(camera.maximumZoomFactor(), 55);
@@ -763,9 +771,9 @@ void tst_QCamera::testSignalExposureCompensationChanged()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy spyExposureCompensationChanged(&camera, SIGNAL(exposureCompensationChanged(float)));
+    QSignalSpy spyExposureCompensationChanged(&camera, &QCamera::exposureCompensationChanged);
 
-    QVERIFY(spyExposureCompensationChanged.size() ==0);
+    QVERIFY(spyExposureCompensationChanged.size() == 0);
 
     QVERIFY(camera.exposureCompensation() != 800);
     camera.setExposureCompensation(2.0);
@@ -790,7 +798,7 @@ void tst_QCamera::testSignalIsoSensitivityChanged()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy spyisoSensitivityChanged(&camera, SIGNAL(isoSensitivityChanged(int)));
+    QSignalSpy spyisoSensitivityChanged(&camera, &QCamera::isoSensitivityChanged);
 
     QVERIFY(spyisoSensitivityChanged.size() ==0);
 
@@ -805,9 +813,9 @@ void tst_QCamera::testSignalShutterSpeedChanged()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy spySignalExposureTimeChanged(&camera, SIGNAL(exposureTimeChanged(float)));
+    QSignalSpy spySignalExposureTimeChanged(&camera, &QCamera::exposureTimeChanged);
 
-    QVERIFY(spySignalExposureTimeChanged.size() ==0);
+    QVERIFY(spySignalExposureTimeChanged.size() == 0);
 
     camera.setManualExposureTime(2.0);//set the ManualShutterSpeed to 2.0
     QTest::qWait(100);
@@ -821,7 +829,7 @@ void tst_QCamera::testSignalFlashReady()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy spyflashReady(&camera,SIGNAL(flashReady(bool)));
+    QSignalSpy spyflashReady(&camera, &QCamera::flashReady);
 
     QVERIFY(spyflashReady.size() == 0);
 
