@@ -1,7 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-
-//TESTED_COMPONENT=src/multimedia
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 #include <QDebug>
@@ -10,8 +8,10 @@
 #include "private/qguiapplication_p.h"
 #include <qmediarecorder.h>
 #include <qaudioformat.h>
-#include <qmockintegration_p.h>
+#include <qmockintegration.h>
 #include <qmediacapturesession.h>
+#include <qscreencapture.h>
+#include <qwindowcapture.h>
 
 #include "qguiapplication_platform.h"
 #include "qmockmediacapturesession.h"
@@ -19,20 +19,31 @@
 
 QT_USE_NAMESPACE
 
+Q_ENABLE_MOCK_MULTIMEDIA_PLUGIN
+
 class tst_QMediaRecorder : public QObject
 {
     Q_OBJECT
 
 public slots:
-    void initTestCase();
-    void cleanupTestCase();
+    void init();
+    void cleanup();
 
 private slots:
     void testBasicSession();
     void testNullControls();
-    void testDeleteMediaSource();
+    void testDeleteMediaCapture();
     void testError();
-    void testSink();
+
+    void record_initializesActualLocation();
+    void record_emitsSignals_whenSettingsChange();
+
+    void setOutputLocation_resetsActualLocation_data();
+    void setOutputLocation_resetsActualLocation();
+
+    void setOutputDevice_resetsActualLocation();
+    void setOutputDevice_doesntChangeActualLocation_whenDeviceIsNull();
+
     void testRecord();
     void testEncodingSettings();
     void testAudioSettings();
@@ -50,32 +61,30 @@ private slots:
     void testApplicationInative();
 
 private:
-    QMockIntegration *mockIntegration = nullptr;
-    QMediaCaptureSession *captureSession;
-    QCamera *object = nullptr;
+    std::unique_ptr<QMediaCaptureSession> captureSession;
+    std::unique_ptr<QCamera> camera;
+    std::unique_ptr<QMediaRecorder> encoder;
+
     QMockMediaCaptureSession *service = nullptr;
-    QMockMediaEncoder *mock;
-    QMediaRecorder *encoder;
+    QMockMediaEncoder *mock = nullptr;
 };
 
-void tst_QMediaRecorder::initTestCase()
+void tst_QMediaRecorder::init()
 {
-    mockIntegration = new QMockIntegration;
-    captureSession = new QMediaCaptureSession;
-    object = new QCamera;
-    encoder = new QMediaRecorder;
-    captureSession->setCamera(object);
-    captureSession->setRecorder(encoder);
-    service = mockIntegration->lastCaptureService();
+    captureSession = std::make_unique<QMediaCaptureSession>();
+    camera = std::make_unique<QCamera>();
+    encoder = std::make_unique<QMediaRecorder>();
+    captureSession->setCamera(camera.get());
+    captureSession->setRecorder(encoder.get());
+    service = QMockIntegration::instance()->lastCaptureService();
     mock = service->mockControl;
 }
 
-void tst_QMediaRecorder::cleanupTestCase()
+void tst_QMediaRecorder::cleanup()
 {
-    delete encoder;
-    delete object;
-    delete captureSession;
-    delete mockIntegration;
+    encoder.reset();
+    camera.reset();
+    captureSession.reset();
 }
 
 void tst_QMediaRecorder::testBasicSession()
@@ -126,7 +135,7 @@ void tst_QMediaRecorder::testNullControls()
     QCOMPARE(recorder.mediaFormat().videoCodec(), QMediaFormat::VideoCodec::VP9);
     QCOMPARE(recorder.mediaFormat().fileFormat(), QMediaFormat::MPEG4);
 
-    QSignalSpy spy(&recorder, SIGNAL(recorderStateChanged(RecorderState)));
+    QSignalSpy spy(&recorder, &QMediaRecorder::recorderStateChanged);
 
     recorder.record();
     QCOMPARE(recorder.recorderState(), QMediaRecorder::StoppedState);
@@ -144,35 +153,59 @@ void tst_QMediaRecorder::testNullControls()
     QCOMPARE(spy.size(), 0);
 }
 
-void tst_QMediaRecorder::testDeleteMediaSource()
+void tst_QMediaRecorder::testDeleteMediaCapture()
 {
     QMediaCaptureSession session;
-    QCamera *camera = new QCamera;
-    QMediaRecorder *recorder = new QMediaRecorder;
-    session.setCamera(camera);
-    session.setRecorder(recorder);
+    QMediaRecorder recorder;
 
-    QVERIFY(session.camera() == camera);
-    QVERIFY(recorder->isAvailable());
+    session.setRecorder(&recorder);
 
-    delete camera;
+    auto checkSourceDeleting = [&](auto setter, auto getter, auto signal) {
+        using Type = std::remove_pointer_t<decltype((session.*getter)())>;
 
-    QVERIFY(session.camera() == nullptr);
-    QVERIFY(recorder->isAvailable());
+        auto errorPrinter = qScopeGuard(
+                []() { qDebug() << QMetaType::fromType<Type>().name() << "deleting failed"; });
 
-    delete recorder;
+        auto capture = std::make_unique<Type>();
+
+        (session.*setter)(capture.get());
+
+        QVERIFY((session.*getter)() == capture.get());
+
+        QSignalSpy spy(&session, signal);
+        capture.reset();
+
+        QCOMPARE(spy.size(), 1);
+        QCOMPARE((session.*getter)(), nullptr);
+
+        QVERIFY(recorder.isAvailable());
+
+        errorPrinter.dismiss();
+    };
+
+    checkSourceDeleting(&QMediaCaptureSession::setImageCapture,
+                        &QMediaCaptureSession::imageCapture,
+                        &QMediaCaptureSession::imageCaptureChanged);
+    checkSourceDeleting(&QMediaCaptureSession::setCamera, &QMediaCaptureSession::camera,
+                        &QMediaCaptureSession::cameraChanged);
+    checkSourceDeleting(&QMediaCaptureSession::setScreenCapture,
+                        &QMediaCaptureSession::screenCapture,
+                        &QMediaCaptureSession::screenCaptureChanged);
+    checkSourceDeleting(&QMediaCaptureSession::setWindowCapture,
+                        &QMediaCaptureSession::windowCapture,
+                        &QMediaCaptureSession::windowCaptureChanged);
 }
 
 void tst_QMediaRecorder::testError()
 {
     const QString errorString(QLatin1String("format error"));
 
-    QSignalSpy spy(encoder, SIGNAL(errorOccurred(Error, const QString&)));
+    QSignalSpy spy(encoder.get(), &QMediaRecorder::errorOccurred);
 
     QCOMPARE(encoder->error(), QMediaRecorder::NoError);
     QCOMPARE(encoder->errorString(), QString());
 
-    mock->error(QMediaRecorder::FormatError, errorString);
+    mock->updateError(QMediaRecorder::FormatError, errorString);
     QCOMPARE(encoder->error(), QMediaRecorder::FormatError);
     QCOMPARE(encoder->errorString(), errorString);
     QCOMPARE(spy.size(), 1);
@@ -180,35 +213,215 @@ void tst_QMediaRecorder::testError()
     QCOMPARE(spy.last()[0].value<QMediaRecorder::Error>(), QMediaRecorder::FormatError);
 }
 
-void tst_QMediaRecorder::testSink()
+void tst_QMediaRecorder::record_initializesActualLocation()
 {
-    encoder->setOutputLocation(QUrl("test.tmp"));
-    QUrl s = encoder->outputLocation();
-    QCOMPARE(s.toString(), QString("test.tmp"));
-    QCOMPARE(encoder->actualLocation(), QUrl());
-
-    //the actual location is available after record
-    encoder->record();
-    QCOMPARE(encoder->actualLocation().toString(), QString("test.tmp"));
-    encoder->stop();
-    QCOMPARE(encoder->actualLocation().toString(), QString("test.tmp"));
-
-    //setOutputLocation resets the actual location
-    encoder->setOutputLocation(QUrl());
-    QCOMPARE(encoder->actualLocation(), QUrl());
-
+    // Since the class uses a mock implementation, the test only verifies that
+    // QMediaEncoder::record doesn't reset actual location after the mock has set it.
     encoder->record();
     QCOMPARE(encoder->actualLocation(), QUrl::fromLocalFile("default_name.mp4"));
-    encoder->stop();
-    QCOMPARE(encoder->actualLocation(), QUrl::fromLocalFile("default_name.mp4"));
+}
 
-    mock->reset();
+void tst_QMediaRecorder::record_emitsSignals_whenSettingsChange()
+{
+    {
+        QSignalSpy mediaFormatChanged{ encoder.get(), &QMediaRecorder::mediaFormatChanged };
+
+        encoder->record();
+        QVERIFY(mediaFormatChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setMediaFormat({ QMediaFormat::Wave });
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(mediaFormatChanged.size(), 1);
+        QCOMPARE_EQ(encoder->mediaFormat(), QMediaFormat::Wave);
+    }
+
+    {
+        QSignalSpy encodingModeChanged{ encoder.get(), &QMediaRecorder::encodingModeChanged };
+
+        encoder->record();
+        QVERIFY(encodingModeChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setEncodingMode(QMediaRecorder::AverageBitRateEncoding);
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(encodingModeChanged.size(), 1);
+        QCOMPARE_EQ(encoder->encodingMode(), QMediaRecorder::AverageBitRateEncoding);
+    }
+
+    {
+        QSignalSpy qualityChanged{ encoder.get(), &QMediaRecorder::qualityChanged };
+
+        encoder->record();
+        QVERIFY(qualityChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setQuality(QMediaRecorder::HighQuality);
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(qualityChanged.size(), 1);
+        QCOMPARE_EQ(encoder->quality(), QMediaRecorder::HighQuality);
+    }
+
+    {
+        QSignalSpy videoResolutionChanged{ encoder.get(), &QMediaRecorder::videoResolutionChanged };
+
+        encoder->record();
+        QVERIFY(videoResolutionChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setVideoResolution({ 64, 64 });
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(videoResolutionChanged.size(), 1);
+        QCOMPARE_EQ(encoder->videoResolution(), QSize(64, 64));
+    }
+
+    {
+        QSignalSpy videoFrameRateChanged{ encoder.get(), &QMediaRecorder::videoFrameRateChanged };
+
+        encoder->record();
+        QVERIFY(videoFrameRateChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setVideoFrameRate(static_cast<qreal>(60));
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(videoFrameRateChanged.size(), 1);
+        QVERIFY(qFuzzyCompare(encoder->videoFrameRate(), 60));
+    }
+
+    {
+        QSignalSpy videoBitRateChanged{ encoder.get(), &QMediaRecorder::videoBitRateChanged };
+
+        encoder->record();
+        QVERIFY(videoBitRateChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setVideoBitRate(128000);
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(videoBitRateChanged.size(), 1);
+        QCOMPARE_EQ(encoder->videoBitRate(), 128000);
+    }
+
+    {
+        QSignalSpy audioBitRateChanged{ encoder.get(), &QMediaRecorder::audioBitRateChanged };
+
+        encoder->record();
+        QVERIFY(audioBitRateChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setAudioBitRate(128000);
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(audioBitRateChanged.size(), 1);
+        QCOMPARE_EQ(encoder->audioBitRate(), 128000);
+    }
+
+    {
+        QSignalSpy audioChannelCountChanged{ encoder.get(),
+                                             &QMediaRecorder::audioChannelCountChanged };
+
+        encoder->record();
+        QVERIFY(audioChannelCountChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setAudioChannelCount(2);
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(audioChannelCountChanged.size(), 1);
+        QCOMPARE_EQ(encoder->audioChannelCount(), 2);
+    }
+
+    {
+        QSignalSpy audioSampleRateChanged{ encoder.get(), &QMediaRecorder::audioSampleRateChanged };
+
+        encoder->record();
+        QVERIFY(audioSampleRateChanged.empty());
+
+        mock->m_settingsModifier = [](QMediaEncoderSettings &settings) {
+            settings.setAudioSampleRate(64000);
+        };
+
+        encoder->record();
+        QCOMPARE_EQ(audioSampleRateChanged.size(), 1);
+        QCOMPARE_EQ(encoder->audioSampleRate(), 64000);
+    }
+}
+
+void tst_QMediaRecorder::setOutputLocation_resetsActualLocation_data()
+{
+    QTest::addColumn<QString>("initialOutputLocation");
+    QTest::addColumn<QString>("newOutputLocation");
+
+    QTest::newRow("empty") << QString() << QString();
+    QTest::newRow("the same non-empty locations") << QString("test.tmp") << QString("test.tmp");
+    QTest::newRow("different locations") << QString("test1.tmp") << QString("test2.tmp");
+}
+
+void tst_QMediaRecorder::setOutputLocation_resetsActualLocation()
+{
+    // Arrange
+    QFETCH(QString, initialOutputLocation);
+    QFETCH(QString, newOutputLocation);
+
+    encoder->setOutputLocation(initialOutputLocation);
+    encoder->record();
+    encoder->stop();
+
+    QCOMPARE_NE(encoder->actualLocation(), QUrl());
+
+    // Act
+    encoder->setOutputLocation(newOutputLocation);
+
+    // Assert
+    QCOMPARE(encoder->actualLocation(), QUrl());
+}
+
+void tst_QMediaRecorder::setOutputDevice_resetsActualLocation()
+{
+    // Arrange
+    encoder->record();
+    encoder->stop();
+
+    QCOMPARE_NE(encoder->actualLocation(), QUrl());
+    QTemporaryFile file;
+
+    // Act
+    encoder->setOutputDevice(&file);
+
+    // Assert
+    QCOMPARE(encoder->actualLocation(), QUrl());
+}
+
+void tst_QMediaRecorder::setOutputDevice_doesntChangeActualLocation_whenDeviceIsNull()
+{
+    // Arrange
+    encoder->record();
+    encoder->stop();
+
+    // Act
+    encoder->setOutputDevice(nullptr);
+
+    // Assert
+    QCOMPARE(encoder->actualLocation(), QUrl::fromLocalFile("default_name.mp4"));
 }
 
 void tst_QMediaRecorder::testRecord()
 {
-    QSignalSpy stateSignal(encoder,SIGNAL(recorderStateChanged(RecorderState)));
-    QSignalSpy progressSignal(encoder, SIGNAL(durationChanged(qint64)));
+    QSignalSpy stateSignal(encoder.get(), &QMediaRecorder::recorderStateChanged);
+    QSignalSpy progressSignal(encoder.get(), &QMediaRecorder::durationChanged);
     encoder->record();
     QCOMPARE(encoder->recorderState(), QMediaRecorder::RecordingState);
     QCOMPARE(encoder->error(), QMediaRecorder::NoError);
@@ -238,8 +451,6 @@ void tst_QMediaRecorder::testRecord()
 
     mock->stop();
     QCOMPARE(stateSignal.size(), 3);
-
-    mock->reset();
 }
 
 void tst_QMediaRecorder::testEncodingSettings()
@@ -310,31 +521,65 @@ void tst_QMediaRecorder::testVideoSettings()
 
     QCOMPARE(recorder.mediaFormat(), QMediaFormat());
     QCOMPARE(recorder.mediaFormat().videoCodec(), QMediaFormat::VideoCodec::Unspecified);
-    QMediaFormat format;
-    format.setVideoCodec(QMediaFormat::VideoCodec::H265);
-    recorder.setMediaFormat(format);
-    QCOMPARE(recorder.mediaFormat(), format);
-    QCOMPARE(recorder.mediaFormat().videoCodec(), QMediaFormat::VideoCodec::H265);
 
-    QCOMPARE(recorder.videoBitRate(), -1);
-    recorder.setVideoBitRate(128000);
-    QCOMPARE(recorder.videoBitRate(), 128000);
+    { // Changing media format
+        QMediaFormat format;
+        format.setVideoCodec(QMediaFormat::VideoCodec::H265);
 
-    QCOMPARE(recorder.quality(), QMediaRecorder::NormalQuality);
-    recorder.setQuality(QMediaRecorder::HighQuality);
-    QCOMPARE(recorder.quality(), QMediaRecorder::HighQuality);
+        QSignalSpy mediaFormatChanged{ &recorder, &QMediaRecorder::mediaFormatChanged };
+        recorder.setMediaFormat(format);
+        QCOMPARE(recorder.mediaFormat(), format);
+        QCOMPARE(recorder.mediaFormat().videoCodec(), QMediaFormat::VideoCodec::H265);
+        QCOMPARE_EQ(mediaFormatChanged.size(), 1);
 
-    QCOMPARE(recorder.videoFrameRate(), -1);
-    recorder.setVideoFrameRate(60);
-    QVERIFY(qFuzzyCompare(recorder.videoFrameRate(), qreal(60)));
-    recorder.setVideoFrameRate(24.0);
-    QVERIFY(qFuzzyCompare(recorder.videoFrameRate(), qreal(24.0)));
+        recorder.setMediaFormat(format);
+        QCOMPARE_EQ(mediaFormatChanged.size(), 1);
+    }
 
-    QCOMPARE(recorder.videoResolution(), QSize());
-    recorder.setVideoResolution(QSize(320,240));
-    QCOMPARE(recorder.videoResolution(), QSize(320,240));
-    recorder.setVideoResolution(800,600);
-    QCOMPARE(recorder.videoResolution(), QSize(800,600));
+    { // Changing video bit rate
+        QSignalSpy videoBitRateChanged{ &recorder, &QMediaRecorder::videoBitRateChanged };
+        QCOMPARE(recorder.videoBitRate(), -1);
+        recorder.setVideoBitRate(128000);
+        QCOMPARE(recorder.videoBitRate(), 128000);
+        QCOMPARE_EQ(videoBitRateChanged.size(), 1);
+
+        recorder.setVideoBitRate(128000);
+        QCOMPARE_EQ(videoBitRateChanged.size(), 1);
+    }
+
+    { // Changing quality
+        QSignalSpy qualityChanged{ &recorder, &QMediaRecorder::qualityChanged };
+        QCOMPARE(recorder.quality(), QMediaRecorder::NormalQuality);
+        recorder.setQuality(QMediaRecorder::HighQuality);
+        QCOMPARE(recorder.quality(), QMediaRecorder::HighQuality);
+        QCOMPARE_EQ(qualityChanged.size(), 1);
+
+        recorder.setQuality(QMediaRecorder::HighQuality);
+        QCOMPARE_EQ(qualityChanged.size(), 1);
+    }
+
+    { // Changing video frame rate
+        QSignalSpy videoFrameRateChanged{ &recorder, &QMediaRecorder::videoFrameRateChanged };
+        QCOMPARE(recorder.videoFrameRate(), -1);
+        recorder.setVideoFrameRate(60);
+        QVERIFY(qFuzzyCompare(recorder.videoFrameRate(), qreal(60)));
+        QCOMPARE_EQ(videoFrameRateChanged.size(), 1);
+
+        recorder.setVideoFrameRate(60);
+        QCOMPARE_EQ(videoFrameRateChanged.size(), 1);
+    }
+
+    { // Changing video resolution
+        QSignalSpy videoResolutionChanged{ &recorder, &QMediaRecorder::videoResolutionChanged };
+
+        QCOMPARE(recorder.videoResolution(), QSize());
+        recorder.setVideoResolution(QSize(320,240));
+        QCOMPARE(recorder.videoResolution(), QSize(320,240));
+        QCOMPARE_EQ(videoResolutionChanged.size(), 1);
+
+        recorder.setVideoResolution(QSize(320, 240));
+        QCOMPARE_EQ(videoResolutionChanged.size(), 1);
+    }
 }
 
 void tst_QMediaRecorder::testSettingsApplied()
@@ -363,10 +608,10 @@ void tst_QMediaRecorder::metaData()
     QVERIFY(recorder.metaData().isEmpty());
 
     QMediaMetaData data;
-    data.insert(QMediaMetaData::Author, QString::fromUtf8("John Doe"));
+    data.insert(QMediaMetaData::Author, QStringLiteral("John Doe"));
     recorder.setMetaData(data);
 
-    QCOMPARE(recorder.metaData().value(QMediaMetaData::Author).toString(), QString::fromUtf8("John Doe"));
+    QCOMPARE(recorder.metaData().value(QMediaMetaData::Author).toString(), QStringLiteral("John Doe"));
 }
 
 void tst_QMediaRecorder::testIsAvailable()
@@ -390,12 +635,12 @@ void tst_QMediaRecorder::testEnum()
 {
     const QString errorString(QLatin1String("resource error"));
 
-    QSignalSpy spy(encoder, SIGNAL(errorOccurred(Error, const QString&)));
+    QSignalSpy spy(encoder.get(), &QMediaRecorder::errorOccurred);
 
     QCOMPARE(encoder->error(), QMediaRecorder::NoError);
     QCOMPARE(encoder->errorString(), QString());
 
-    emit mock->error(QMediaRecorder::ResourceError, errorString);
+    mock->updateError(QMediaRecorder::ResourceError, errorString);
     QCOMPARE(encoder->error(), QMediaRecorder::ResourceError);
     QCOMPARE(encoder->errorString(), errorString);
     QCOMPARE(spy.size(), 1);
@@ -448,7 +693,7 @@ void tst_QMediaRecorder::testApplicationInative()
     encoder.setQuality(QMediaRecorder::VeryHighQuality);
 
     encoder.setOutputLocation(QUrl("test.tmp"));
-    QCOMPARE(encoder.outputLocation().toString(), QString("test.tmp"));
+    QCOMPARE(encoder.outputLocation().toString(), QStringLiteral("test.tmp"));
     QCOMPARE(encoder.actualLocation(), QUrl());
 
     encoder.record();
@@ -462,7 +707,7 @@ void tst_QMediaRecorder::testApplicationInative()
     encoder.stop();
 
     // the actual location is available after record
-    QCOMPARE(encoder.actualLocation().toString(), QString("test.tmp"));
+    QCOMPARE(encoder.actualLocation().toString(), QStringLiteral("test.tmp"));
 }
 
 QTEST_GUILESS_MAIN(tst_QMediaRecorder)
